# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.core.urlresolvers import NoReverseMatch, reverse
from django.db import models
from django.utils import timezone
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from django_fsm import transition
from shop.models.order import BaseOrder, BaseOrderItem
from shop.models.order import OrderManager as OrderManagerBase


class OrderManager(OrderManagerBase):
    def get_summary_url(self):
        try:
            return reverse('shopit-account-order-list')
        except NoReverseMatch:
            return super(OrderManager, self).get_summary_url()

    def get_latest_url(self):
        try:
            return reverse('shopit-account-order-latest')
        except NoReverseMatch:
            return reverse('shopit-thanks')
        except NoReverseMatch:
            return super(OrderManager, self).get_latest_url()


@python_2_unicode_compatible
class Order(BaseOrder):
    number = models.PositiveIntegerField(
        _("Order Number"),
        null=True,
        default=None,
        unique=True,
    )

    shipping_address_text = models.TextField(
        _("Shipping Address"),
        blank=True,
        null=True,
        help_text=_('Shipping address at the moment of purchase.'),
    )

    billing_address_text = models.TextField(
        _("Billing Address"),
        blank=True,
        null=True,
        help_text=_('Billing address at the moment of purchase.'),
    )

    objects = OrderManager()

    class Meta:
        db_table = 'shopit_orders'
        verbose_name = _('Order')
        verbose_name_plural = _('Orders')

    def __str__(self):
        return self.get_number() if self.number else str(self.pk)

    def get_absolute_url(self):
        try:
            return reverse('shopit-account-order-detail', args=[self.pk])
        except NoReverseMatch:
            pass

    def get_or_assign_number(self):
        """
        Set a unique number to identify this Order object. The first 4 digits
        represent the current year. The last five digits represent a
        zero-padded incremental counter.
        """
        if self.number is None:
            epoch = timezone.now().date()
            epoch = epoch.replace(epoch.year, 1, 1)
            aggr = Order.objects.filter(number__isnull=False, created_at__gt=epoch).aggregate(models.Max('number'))
            try:
                epoc_number = int(str(aggr['number__max'])[4:]) + 1
                self.number = int('{0}{1:05d}'.format(epoch.year, epoc_number))
            except (KeyError, ValueError):
                # the first order this year
                self.number = int('{0}00001'.format(epoch.year))
        return self.get_number()

    def get_number(self):
        return str(self.number)[:4] + '-' + str(self.number)[4:]

    @classmethod
    def resolve_number(cls, number):
        number = number[:4] + number[5:]
        return dict(number=number)

    def populate_from_cart(self, cart, request):
        self.shipping_address_text = cart.shipping_address.as_text() if cart.shipping_address else ''
        self.billing_address_text = cart.billing_address.as_text() if cart.billing_address else ''

        # in case one of the addresses was None, the customer presumably intended the other one.
        if not self.shipping_address_text:
            self.shipping_address_text = self.billing_address_text
        if not self.billing_address_text:
            self.billing_address_text = self.shipping_address_text
        super(Order, self).populate_from_cart(cart, request)

        # After order was populated with cart data, delete discount codes.
        cart.delete_discount_codes()

    def is_fully_paid(self):
        return super(Order, self).is_fully_paid()

    @transition(field='status', source='*', target='payment_confirmed', conditions=[is_fully_paid],
                custom=dict(admin=False))
    def acknowledge_payment(self, by=None):
        """
        This is here only to force setting `admin=False` to hide the
        button in admin.
        """


@python_2_unicode_compatible
class OrderItem(BaseOrderItem):
    quantity = models.IntegerField(_('Ordered quantity'))

    canceled = models.BooleanField(
        _('Item canceled'),
        default=False,
    )

    class Meta:
        db_table = 'shopit_order_items'
        verbose_name = _('Order item')
        verbose_name_plural = _('Order items')

    def __str__(self):
        return self.product_name

    def populate_from_cart_item(self, cart_item, request):
        self.product_code = cart_item.product.product_code
        super(OrderItem, self).populate_from_cart_item(cart_item, request)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-06-26 12:34
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shopit', '0007_make_reviews_non_translatable'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='phone_number',
            field=models.CharField(blank=True, max_length=128, verbose_name='Phone number'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-04-10 13:25
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('shopit', '0005_add_product_code_to_cart_item'),
    ]

    operations = [
        migrations.RenameField(
            model_name='producttranslation',
            old_name='caption',
            new_name='_caption',
        ),
        migrations.RenameField(
            model_name='producttranslation',
            old_name='description',
            new_name='_description',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.contrib import admin
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _
from mptt.admin import DraggableMPTTAdmin
from parler.admin import TranslatableAdmin

from shopit.forms.flag import FlagModelForm
from shopit.models.flag import Flag


@admin.register(Flag)
class FlagAdmin(TranslatableAdmin, DraggableMPTTAdmin):
    form = FlagModelForm
    list_display = ['tree_actions', 'get_name', 'code', 'active', 'language_column']
    list_display_links = ['get_name']
    readonly_fields = ['created_at', 'updated_at']

    fieldsets = [
        (_('Basic info'), {'fields': ['name', 'code']}),
        (_('Status'), {'fields': [('active', 'created_at', 'updated_at')]}),
        (_('Settings'), {'fields': ['parent', 'template']}),
    ]

    class Media:
        css = {'all': ['shopit/css/djangocms-admin-style.css']}

    def get_prepopulated_fields(self, request, obj=None):
        return {'code': ['name']}  # pragma: no cover

    def get_name(self, obj):
        return format_html(
            '<div style="text-indent:{}px">{}</div>',
            obj.level * self.mptt_level_indent, obj.safe_translation_getter('name', any_language=True))
    get_name.short_description = _('Name')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _
from django.views.generic import RedirectView

from shopit import views
from shopit.conf import app_settings


def get_urls(name, prefixed=app_settings.SINGLE_APPHOOK):  # noqa
    """
    Returns url patterns for the given module.
    Checks if urls are handled with single or multiple apphooks.
    """
    pk_regexp = r'(?P<pk>\d+)'
    slug_regexp = r'(?P<slug>[-_\w\d]+)'
    path_regexp = r'(?P<path>[-_/\w\d]+)'
    list_regexp = r'^$'
    detail_regexp = r'^%s/$' % slug_regexp

    if name == 'shop':
        cart_regexp = _(r'cart/$')
        cart_empty_regexp = _(r'cart/empty/$')
        watch_regexp = _(r'watch/$')
        checkout_regexp = _(r'checkout/$')
        thanks_regexp = _(r'thanks/$')

        if prefixed:
            list_regexp = _(r'shop/$')
            cart_regexp = _(r'shop/cart/$')
            cart_empty_regexp = _(r'shop/cart/empty/$')
            watch_regexp = _(r'shop/watch/$')
            checkout_regexp = _(r'shop/checkout/$')
            thanks_regexp = _(r'shop/thanks/$')

        return [
            url(list_regexp, RedirectView.as_view(pattern_name='shopit-cart')),
            url(cart_regexp, views.CartView.as_view(), name='shopit-cart'),
            url(cart_empty_regexp, views.CartView.as_view(empty=True), name='shopit-cart-empty'),
            url(watch_regexp, views.WatchView.as_view(), name='shopit-watch'),
            url(checkout_regexp, views.CheckoutView.as_view(), name='shopit-checkout'),
            url(thanks_regexp, views.ThanksView.as_view(), name='shopit-thanks'),
        ]

    if name == 'account':
        reset_confirm_kwargs = r'(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})'

        detail_regexp = list_regexp
        login_regexp = _(r'^login/$')
        logout_regexp = _(r'^logout/$')
        register_regexp = _(r'^register/$')
        reset_regexp = _(r'^reset/$')
        reset_confirm_regexp = _(r'^reset/confirm/%s/$') % reset_confirm_kwargs
        order_list_regexp = _(r'^orders/$')
        order_detail_regexp = _(r'^orders/%s/$') % pk_regexp
        order_latest_regexp = _(r'^orders/latest/$')
        settings_regexp = _(r'^settings/$')
        settings_details_regexp = _(r'^settings/details/$')
        settings_password_regexp = _(r'^settings/password/$')

        if prefixed:
            detail_regexp = _(r'^account/$')
            login_regexp = _(r'^account/login/$')
            logout_regexp = _(r'^account/logout/$')
            register_regexp = _(r'^account/register/$')
            reset_regexp = _(r'^account/reset/$')
            reset_confirm_regexp = _(r'^account/reset/confirm/%s/$') % reset_confirm_kwargs
            order_list_regexp = _(r'^account/orders/$')
            order_detail_regexp = _(r'^account/orders/%s/$') % pk_regexp
            order_latest_regexp = _(r'^account/orders/latest/$')
            settings_regexp = _(r'^account/settings/$')
            settings_details_regexp = _(r'^account/settings/details/$')
            settings_password_regexp = _(r'^account/settings/password/$')

        return [
            url(detail_regexp, views.AccountDetailView.as_view(), name='shopit-account-detail'),
            url(login_regexp, views.AccountLoginView.as_view(), name='shopit-account-login'),
            url(logout_regexp, views.AccountLogoutView.as_view(), name='shopit-account-logout'),
            url(register_regexp, views.AccountRegisterView.as_view(), name='shopit-account-register'),
            url(reset_regexp, views.AccountResetView.as_view(), name='shopit-account-reset'),
            url(reset_confirm_regexp, views.AccountResetConfirmView.as_view(), name='shopit-account-reset-confirm'),
            url(order_list_regexp, views.AccountOrderView.as_view(), name='shopit-account-order-list'),
            url(order_detail_regexp, views.AccountOrderView.as_view(many=False), name='shopit-account-order-detail'),
            url(order_latest_regexp, views.AccountOrderView.as_view(latest=True), name='shopit-account-order-latest'),
            url(settings_regexp, views.AccountSettingsView.as_view(), name='shopit-account-settings'),
            url(settings_details_regexp, views.AccountSettingsView.as_view(action='details'),
                name='shopit-account-settings-details'),
            url(settings_password_regexp, views.AccountSettingsView.as_view(action='password'),
                name='shopit-account-settings-password'),
        ]

    if name == 'categories':
        detail_regexp = r'^%s/$' % path_regexp

        if prefixed:
            list_regexp = _(r'^categories/$')
            detail_regexp = _(r'^categories/%s/$') % path_regexp

        return [
            url(list_regexp, views.CategoryListView.as_view(), name='shopit-category-list'),
            url(detail_regexp, views.CategoryDetailView.as_view(), name='shopit-category-detail'),
        ]

    if name == 'brands':
        detail_regexp = r'^%s/$' % path_regexp

        if prefixed:
            list_regexp = _(r'^brands/$')
            detail_regexp = _(r'^brands/%s/$') % path_regexp

        return [
            url(list_regexp, views.BrandListView.as_view(), name='shopit-brand-list'),
            url(detail_regexp, views.BrandDetailView.as_view(), name='shopit-brand-detail'),
        ]

    if name == 'manufacturers':
        detail_regexp = r'^%s/$' % path_regexp

        if prefixed:
            list_regexp = _(r'^manufacturers/$')
            detail_regexp = _(r'^manufacturers/%s/$') % path_regexp

        return [
            url(list_regexp, views.ManufacturerListView.as_view(), name='shopit-manufacturer-list'),
            url(detail_regexp, views.ManufacturerDetailView.as_view(), name='shopit-manufacturer-detail'),
        ]

    if name == 'products':
        review_regexp = _(r'^%s/reviews/$') % slug_regexp
        review_detail_regexp = _(r'^%s/reviews/%s/$') % (slug_regexp, pk_regexp)
        add_to_cart_regexp = _(r'^%s/add-to-cart/$') % slug_regexp

        if prefixed:
            list_regexp = _(r'^products/$')
            detail_regexp = _(r'^products/%s/$') % slug_regexp
            review_regexp = _(r'^products/%s/reviews/$') % slug_regexp
            review_detail_regexp = _(r'^products/%s/reviews/%s/$') % (slug_regexp, pk_regexp)
            add_to_cart_regexp = _(r'^products/%s/add-to-cart/$') % slug_regexp

        return [
            url(list_regexp, views.ProductListView.as_view(), name='shopit-product-list'),
            url(detail_regexp, views.ProductDetailView.as_view(), name='shopit-product-detail'),
            url(review_regexp, views.ProductReviewListView.as_view(), name='shopit-product-review-list'),
            url(review_detail_regexp, views.ProductReviewDetailView.as_view(), name='shopit-product-review-detail'),
            url(add_to_cart_regexp, views.AddToCartView.as_view(), name='shopit-add-to-cart'),
        ]


urlpatterns = []
for name in ('shop', 'account', 'categories', 'brands', 'manufacturers', 'products'):
    urlpatterns.extend(get_urls(name, prefixed=True))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-06-06 13:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shopit', '0006_rename_caption_description'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='reviewtranslation',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='reviewtranslation',
            name='master',
        ),
        migrations.AddField(
            model_name='review',
            name='language',
            field=models.CharField(choices=[('en', 'English')], default='en', max_length=2, verbose_name='Language'),
        ),
        migrations.AddField(
            model_name='review',
            name='name',
            field=models.CharField(blank=True, max_length=128, verbose_name='Name'),
        ),
        migrations.AddField(
            model_name='review',
            name='text',
            field=models.TextField(default='', max_length=1024, verbose_name='Text'),
            preserve_default=False,
        ),
        migrations.DeleteModel(
            name='ReviewTranslation',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from adminsortable2.admin import SortableAdminMixin, SortableInlineAdminMixin
from cms.admin.placeholderadmin import FrontendEditableAdminMixin, PlaceholderAdminMixin
from cms.utils.i18n import get_current_language
from django.conf.urls import url
from django.contrib import admin, messages
from django.contrib.admin.views.main import ChangeList
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.db import transaction
from django.db.utils import IntegrityError
from django.http import HttpResponseBadRequest, HttpResponseRedirect, JsonResponse
from django.shortcuts import get_object_or_404
from django.template.loader import render_to_string
from django.utils.decorators import method_decorator
from django.utils.encoding import smart_text
from django.utils.formats import date_format
from django.utils.html import format_html
from django.utils.http import urlencode
from django.utils.translation import ugettext_lazy as _
from mptt.admin import TreeRelatedFieldListFilter
from parler.admin import TranslatableAdmin, TranslatableTabularInline

from shopit.forms.product import (AttributeChoiceInlineFormSet, AttributeValueInlineFormSet, AttributeValueModelForm,
                                  ProductModelForm)
from shopit.models.product import Attachment, Attribute, AttributeChoice, AttributeValue, Product, Relation, Review


class AttributeChoiceInline(SortableInlineAdminMixin, TranslatableTabularInline, admin.TabularInline):
    model = AttributeChoice
    formset = AttributeChoiceInlineFormSet
    fields = ['name', 'value', 'file', 'order']
    extra = 0


@admin.register(Attribute)
class AttributeAdmin(SortableAdminMixin, TranslatableAdmin):
    list_display = ['get_name', 'code', 'nullable', 'active', 'language_column']
    readonly_fields = ['created_at', 'updated_at']
    search_fields = ['code', 'translations__name', 'template']

    fieldsets = [
        (_('Basic info'), {'fields': ['name', 'code']}),
        (_('Status'), {'fields': [('active', 'created_at', 'updated_at')]}),
        (_('Settings'), {'fields': ['template', 'nullable']}),
    ]

    inlines = [AttributeChoiceInline]

    class Media:
        css = {'all': ['shopit/css/djangocms-admin-style.css']}

    def get_prepopulated_fields(self, request, obj=None):
        return {'code': ['name']}  # pragma: no cover

    def get_name(self, obj):
        return str(obj)
    get_name.admin_order_field = 'translations__name'
    get_name.short_description = _('Name')

    def get_template(self, obj):
        return obj.template or None
    get_template.admin_order_field = 'template'
    get_template.short_description = _('Template')


class AttributeValueInline(admin.TabularInline):
    model = AttributeValue
    form = AttributeValueModelForm
    formset = AttributeValueInlineFormSet
    extra = 0


class AttachmentInline(SortableInlineAdminMixin, admin.TabularInline):
    model = Attachment
    extra = 0


class RelationInline(SortableInlineAdminMixin, admin.TabularInline):
    model = Relation
    fk_name = 'base'
    raw_id_fields = ['product']
    extra = 0


class ReviewInline(SortableInlineAdminMixin, admin.TabularInline):
    model = Review
    extra = 0
    fields = ['customer', 'name', 'text', 'rating', 'active']
    readonly_fields = ['customer', 'name', 'text', 'rating', 'created_at']

    def has_add_permission(self, request):
        return False  # pragma: no cover


class ProductChangeList(ChangeList):
    """
    Override ChangeList to filter out Variant products from admin list
    but include them in search results.
    """
    def get_queryset(self, request):
        qs = super(ProductChangeList, self).get_queryset(request)
        if request.GET.get('q'):
            return qs.order_by('-order', 'kind', 'published')
        return qs.filter(kind__in=[Product.SINGLE, Product.GROUP])


@admin.register(Product)
class ProductAdmin(FrontendEditableAdminMixin, PlaceholderAdminMixin, TranslatableAdmin):
    form = ProductModelForm
    list_display = ['get_name', 'get_slug', 'code', 'get_is_group', 'get_unit_price', 'get_discount_percent',
                    'get_tax_percent', 'get_price', 'get_published', 'active', 'language_column']

    list_filter = ['published',
                   ('_category', TreeRelatedFieldListFilter),
                   ('_brand', TreeRelatedFieldListFilter),
                   ('_manufacturer', TreeRelatedFieldListFilter)]

    search_fields = ['code', 'translations__name']
    raw_id_fields = ['group']
    readonly_fields = ['created_at', 'updated_at', 'get_summary_field', 'get_variants_field']
    filter_horizontal = ['modifiers', 'flags', 'available_attributes']

    frontend_editable_fields = [
        'name', 'slug', 'code', 'active', 'created_at', 'updated_at', 'published', '_caption', '_description',
        '_category', '_brand', '_manufacturer', '_unit_price', '_discount', '_tax', 'quantity', 'kind', 'discountable',
        'modifiers', 'flags', '_width', '_height', '_depth', '_weight', 'available_attributes', 'group']

    fieldsets = [
        (_('Basic info'), {'fields': ['name', 'slug', 'code']}),
        (_('Status'), {'fields': [('active', 'created_at', 'updated_at'), 'published']}),
        (_('Description'), {'fields': ['_caption', '_description']}),
        (_('Categorization'), {
            'fields': ['_category', '_brand', '_manufacturer'],
            'description': _('This section is ment for Single and Group products. Variants will inherit their '
                             'categorization from their Group.')}),
        (_('Pricing'), {'fields': ['_unit_price', '_discount', '_tax', 'get_summary_field']}),
        (_('Stock'), {'fields': ['quantity']}),
        (_('Settings'), {'fields': ['kind', 'discountable', 'modifiers', 'flags']}),
        (_('Measurements'), {
            'fields': [('_width', '_height'), ('_depth', '_weight')],
            'description': _('Measurements of a product, mostly used for delivery calculations. Variants can leave '
                             'this section empty to inherit the values from their Group.')}),
        (_('Group'), {'fields': ['available_attributes']}),
        (_('Variants'), {'fields': ['group', 'get_variants_field']}),
    ]

    inlines = [AttributeValueInline, AttachmentInline, RelationInline, ReviewInline]
    actions = ['make_active', 'make_inactive']

    class Media:
        css = {'all': ['shopit/css/djangocms-admin-style.css']}
        js = ['shopit/js/product_admin.js']

    def get_prepopulated_fields(self, request, obj=None):
        return {'slug': ['name']}  # pragma: no cover

    def get_changelist(self, request, **kwargs):
        return ProductChangeList

    def get_changeform_initial_data(self, request):
        initial = super(ProductAdmin, self).get_changeform_initial_data(request)
        try:
            latest = Product.objects.latest('pk')
            initial['code'] = latest.pk + 1
        except Product.DoesNotExist:
            initial['code'] = 1
        return initial

    def get_urls(self):
        urls = super(ProductAdmin, self).get_urls()  # pragma: no cover
        return [
            url(r'^get-attribute-choices/$', self.admin_site.admin_view(self.get_attribute_choices),
                name='shopit_product_get_attribute_choices'),
            url(r'^(?P<pk>\d+)/add-variant/$', self.admin_site.admin_view(self.add_variant),
                name='shopit_product_add_variant'),
            url(r'^(?P<pk>\d+)/create-variant/(?P<combo>\d+)/$', self.admin_site.admin_view(self.create_variant),
                name='shopit_product_create_variant'),
            url(r'^(?P<pk>\d+)/delete-variant/(?P<variant>\d+)/$', self.admin_site.admin_view(self.delete_variant),
                name='shopit_product_delete_variant'),
            url(r'^(?P<pk>\d+)/create-all-variants/$', self.admin_site.admin_view(self.create_all_variants),
                name='shopit_product_create_all_variants'),
            url(r'^(?P<pk>\d+)/delete-invalid-variants/$', self.admin_site.admin_view(self.delete_invalid_variants),
                name='shopit_product_delete_invalid_variants'),
        ] + urls  # pragma: no cover

    def get_name(self, obj):
        if obj.is_variant:
            return '--- %s' % smart_text(obj)
        return smart_text(obj)
    get_name.admin_order_field = 'translations__name'
    get_name.short_description = _('Name')

    def get_slug(self, obj):
        return obj.safe_translation_getter('slug')
    get_slug.allow_tags = True
    get_slug.admin_order_field = 'translations__slug'
    get_slug.short_description = _('Slug')

    def get_is_group(self, obj):
        return obj.is_group
    get_is_group.boolean = True
    get_is_group.admin_order_field = 'kind'
    get_is_group.short_description = _('Group')

    def get_unit_price(self, obj):
        return str(obj.unit_price)
    get_unit_price.short_description = _('Unit price')

    def get_discount_percent(self, obj):
        return '%g%%' % obj.discount_percent
    get_discount_percent.admin_order_field = 'discount'
    get_discount_percent.short_description = _('Discount %')

    def get_tax_percent(self, obj):
        return '%g%%' % obj.tax_percent
    get_tax_percent.admin_order_field = 'tax__percent'
    get_tax_percent.short_description = _('Tax %')

    def get_price(self, obj):
        return str(obj.price)
    get_price.short_description = _('Price')

    def get_published(self, obj):
        return date_format(obj.published, 'SHORT_DATETIME_FORMAT')
    get_published.admin_order_field = 'order'
    get_published.short_description = _('Published')

    def get_summary_field(self, obj):
        unit_price = '%s: %s' % (_('Unit price'), smart_text(obj.unit_price))
        discount = '%s: %g%%' % (_('Discount percent'), obj.discount_percent)
        tax = '%s: %g%%' % (_('Tax percent'), obj.tax_percent)
        price = '<strong>%s: %s</strong>' % (_('Price'), smart_text(obj.price))
        return format_html('<br>'.join([unit_price, discount, tax, price]))
    get_summary_field.allow_tags = True
    get_summary_field.short_description = _('Summary')

    def get_variants_field(self, obj):
        if not obj.pk:
            return None
        return render_to_string('admin/shopit/product_variants_field.html', {
            'product': obj.group or obj,
            'variant': obj if obj.group else None,
        })
    get_variants_field.allow_tags = True
    get_variants_field.short_description = _('Variants')

    def add_variant(self, request, pk):
        product = get_object_or_404(Product, pk=pk)
        product = product.group or product
        variants = product.get_variants()
        num = variants.count() + 1 if variants else 1
        name = '%s #%s' % (product.product_name, num)
        data = {'name': name, 'kind': Product.VARIANT, 'group': product.pk}
        return HttpResponseRedirect('{}?{}'.format(reverse('admin:shopit_product_add'), urlencode(data)))

    def create_variant(self, request, pk, combo, message=True, product=None, language=None):
        """
        This view creates a full variant with it's attribute values
        based on a combination index passed in as `combo`.
        """
        product = product or get_object_or_404(Product, pk=pk)
        product = product.group or product
        if not language:
            language = get_current_language()  # pragma: no cover
        try:
            combo = product.get_combinations()[int(combo)]
            variant = product.create_variant(combo, language=language)
        except (IndexError, ObjectDoesNotExist, IntegrityError):
            return HttpResponseBadRequest()
        if message:
            messages.success(request, _('Variant successfully created.'))  # pragma: no cover
        return HttpResponseRedirect(
            reverse('admin:shopit_product_change', args=[variant.pk]) + '?language=%s' % language)

    def delete_variant(self, request, pk, variant, message=True, product=None):
        product = product or get_object_or_404(Product, pk=pk)
        product = product.group or product
        variant = get_object_or_404(product.get_variants(), pk=variant)
        variant.delete()
        if message:
            messages.success(request, _('Variant successfully deleted.'))  # pragma: no cover
        return HttpResponseRedirect(reverse('admin:shopit_product_change', args=[product.pk]))

    def create_all_variants(self, request, pk, language=None):
        product = get_object_or_404(Product, pk=pk)
        product = product.group or product
        product.create_all_variants(language=language)
        messages.success(request, _('Variants successfully created.'))
        return HttpResponseRedirect(reverse('admin:shopit_product_change', args=[product.pk]))

    @method_decorator(transaction.atomic)
    def delete_invalid_variants(self, request, pk):
        product = get_object_or_404(Product, pk=pk)
        product = product.group or product
        for invalid in product.get_invalid_variants():
            invalid.delete()
        messages.success(request, _('Invalid variants successfully deleted.'))
        return HttpResponseRedirect(reverse('admin:shopit_product_change', args=[product.pk]))

    def get_attribute_choices(self, request):
        """
        A Json response view that's used to fetch the attribute choices
        and update the attribute values select field on a product variant.
        """
        attribute_id = request.GET.get('attribute_id', None) or None
        try:
            attribute = Attribute.objects.get(id=attribute_id)
            choices = [{'id': x.pk, 'label': str(x)} for x in attribute.get_choices()]
        except Attribute.DoesNotExist:
            choices = []
        return JsonResponse({'choices': choices})

    def make_active(self, request, queryset):
        rows = queryset.update(active=True)
        if rows == 1:
            msg = _('1 Product was successfully marked as active.')  # pragma: no cover
        else:
            msg = _('%s Products were successfully marked as active.') % rows
        self.message_user(request, msg, messages.SUCCESS)
    make_active.short_description = _('Mark selected Products as active')

    def make_inactive(self, request, queryset):
        rows = queryset.update(active=False)
        if rows == 1:
            msg = _('1 Product was successfully marked as inactive.')  # pragma: no cover
        else:
            msg = _('%s Products were successfully marked as inactive.') % rows
        self.message_user(request, msg, messages.SUCCESS)
    make_inactive.short_description = _('Mark selected Products as inactive')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from decimal import Decimal

from django.core.validators import MinValueValidator
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from parler.models import TranslatableModel, TranslatedFields


@python_2_unicode_compatible
class Tax(TranslatableModel):
    translations = TranslatedFields(
        name=models.CharField(
            _('Name'),
            max_length=128,
        ),
    )

    percent = models.DecimalField(
        _('Percent'),
        max_digits=4,
        decimal_places=2,
        validators=[MinValueValidator(Decimal('0.00'))],
        help_text=_('Tax percentage.'),
    )

    order = models.PositiveIntegerField(
        _('Sort'),
        default=0,
    )

    class Meta:
        db_table = 'shopit_taxes'
        verbose_name = _('Tax')
        verbose_name_plural = _('Taxes')
        ordering = ['order']

    def __str__(self):
        return self.safe_translation_getter('name', any_language=True)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from adminsortable2.admin import SortableAdminMixin
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from parler.admin import TranslatableAdmin

from shopit.models.tax import Tax


@admin.register(Tax)
class TaxAdmin(SortableAdminMixin, TranslatableAdmin):
    list_display = ['name', 'percent', 'language_column']
    fieldsets = [(_('Basic info'), {'fields': ['name', 'percent']})]

    class Media:
        css = {'all': ['shopit/css/djangocms-admin-style.css']}



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from mptt.forms import MPTTAdminForm
from parler.forms import TranslatableModelForm

from shopit.models.flag import Flag


class FlagModelForm(MPTTAdminForm, TranslatableModelForm):
    class Meta:
        model = Flag
        exclude = []



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django import forms
from django.contrib.auth import get_user_model
from django.forms.utils import ErrorDict
from django.utils.translation import ugettext_lazy as _
from shop.conf import app_settings as shop_settings
from shop.modifiers.pool import cart_modifiers_pool

from shopit.conf import app_settings
from shopit.forms.account import AccountDetailsForm, CleanEmailMixin
from shopit.models.address import ISO_3166_CODES, BillingAddress, ShippingAddress
from shopit.models.cart import CartDiscountCode
from shopit.models.customer import Customer
from shopit.models.modifier import DiscountCode
from shopit.utils import get_error_message as em


class CartDiscountCodeForm(forms.ModelForm):
    """
    Form that handles entering a cart modifier code.
    """
    _discount_code = None

    class Meta:
        model = CartDiscountCode
        fields = ['code']

    def __init__(self, *args, **kwargs):
        self.cart = kwargs.pop('cart')
        kwargs['instance'] = CartDiscountCode(cart=self.cart)
        super(CartDiscountCodeForm, self).__init__(*args, **kwargs)
        self.fields['code'].required = False
        self.fields['code'].label = _('Discount code')

    def clean_code(self):
        code = self.cleaned_data.get('code', None)
        if code:
            cart_codes = self.cart.get_discount_codes().values_list('code', flat=True)
            if code in cart_codes:
                raise forms.ValidationError(em('cart_discount_code_exists'))
            try:
                dc = DiscountCode.objects.valid().get(code=code)
            except DiscountCode.DoesNotExist:
                raise forms.ValidationError(em('cart_discount_code_invalid'))
            if dc.customer and code not in self.cart.customer.get_discount_codes().values_list('code', flat=True):
                raise forms.ValidationError(em('cart_discount_code_wrong_customer'))
            self._discount_code = dc
        return code

    def save(self, commit=True):
        if self._discount_code is not None:
            self._discount_code.use()  # increment `num_uses` field on DiscountCode.
            return super(CartDiscountCodeForm, self).save(commit)


class CheckoutFormMixin(object):
    """
    Checkout form mixin ensures request and cart are passed in.
    """
    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request')
        self.cart = kwargs.pop('cart')
        super(CheckoutFormMixin, self).__init__(*args, **kwargs)


class CustomerForm(CheckoutFormMixin, AccountDetailsForm):
    def __init__(self, *args, **kwargs):
        self.cart = kwargs.pop('cart')
        return AccountDetailsForm.__init__(self, *args, **kwargs)

    def save(self, commit=True):
        self.instance.recognize_as_registered()
        return super(CustomerForm, self).save(commit)


class GuestForm(CheckoutFormMixin, CleanEmailMixin, forms.ModelForm):
    email = forms.EmailField(label=_('Email address'))
    phone_number = forms.CharField(label=_('Phone number'))

    class Meta:
        model = get_user_model()
        fields = ['email']

    def __init__(self, *args, **kwargs):
        super(GuestForm, self).__init__(*args, **kwargs)
        self.customer = Customer.objects.get_from_request(self.request)
        self.instance = self.customer.user
        self.fields['email'].initial = self.instance.email
        self.fields['phone_number'].initial = self.customer.phone_number
        self.fields['phone_number'].required = app_settings.PHONE_NUMBER_REQUIRED

    def save(self, commit=True):
        self.customer.recognize_as_guest()
        self.instance.is_active = shop_settings.SHOP_GUEST_IS_ACTIVE_USER
        if self.instance.is_active:
            password = get_user_model().objects.make_random_password(length=30)
            self.instance.set_password(password)
        self.customer.phone_number = self.cleaned_data.get('phone_number', '')
        self.customer.save()
        return super(GuestForm, self).save(commit)


class AddressForm(CheckoutFormMixin, forms.ModelForm):
    priority = forms.IntegerField(
        required=False,
        widget=forms.HiddenInput,
    )

    existant = forms.ModelChoiceField(
        required=False,
        queryset=None,
        label=_('Use existant address'),
    )

    # Field decides if a primary address should be used instead.
    # Primary address is set to either 'shipping' or 'billing' using `PRIMARY_ADDRESS` setting.
    use_primary_address = forms.BooleanField(
        required=False,
        initial=True,
    )

    class Meta:
        exclude = ['customer']

    def __init__(self, *args, **kwargs):
        self.field_order = ['existant']  # Place `existant` field at the top.
        super(AddressForm, self).__init__(*args, **kwargs)
        self.customer = Customer.objects.get_from_request(self.request)

        # Set existant addresses choices.
        addresses = self.Meta.model.objects.filter(customer=self.customer).order_by('-priority')
        self.fields['existant'].queryset = addresses
        if not addresses.exists():
            self.fields['existant'].widget = forms.HiddenInput()

        # Set country choices based on `ADDRESS_COUNTRIES` setting.
        if app_settings.ADDRESS_COUNTRIES:
            countries = [('', '---------')] + [x for x in ISO_3166_CODES if x in app_settings.ADDRESS_COUNTRIES]
            self.fields['country'].widget = forms.Select(choices=countries)
            self.fields['country'].choices = countries

        if self.is_primary:
            self.fields.pop('use_primary_address')  # remove field from primary address.
        else:
            self.fields['use_primary_address'].initial = \
                getattr(self.cart, '%s_address' % self.address_type, None) is None
            if hasattr(self, 'use_primary_address_label'):
                self.fields['use_primary_address'].label = self.use_primary_address_label

        # If current address is set to the cart, use it as existant one.
        cart_address = getattr(self.cart, '%s_address' % self.address_type, None)
        if cart_address:
            self.fields['existant'].initial = cart_address
            for fname in [f.name for f in cart_address._meta.get_fields() if f.name in self.fields]:
                self.fields[fname].initial = getattr(cart_address, fname, '')

    def full_clean(self):
        super(AddressForm, self).full_clean()
        if not self.is_primary:
            if self.is_bound and self['use_primary_address'].value():
                self._errors = ErrorDict()

    def is_valid(self):
        if not self.is_primary:
            return self['use_primary_address'].value() or super(AddressForm, self).is_valid()
        return super(AddressForm, self).is_valid()

    def clean(self):
        existant = self.cleaned_data['existant']
        if existant:
            self.instance = existant  # Set existant as an instance if selected.
            self.cleaned_data['priority'] = existant.priority
            # Populate missing fields in `cleaned_data` with existant data and skip validation.
            for field in [x for x in self.fields if x not in self.cleaned_data]:
                self.cleaned_data[field] = getattr(existant, field)
                del self._errors[field]
        else:
            self.cleaned_data['priority'] = self.Meta.model.objects.get_max_priority(self.customer) + 1
        return super(AddressForm, self).clean()

    def save(self, commit=True):
        if self.is_primary or not self['use_primary_address'].value():
            instance = super(AddressForm, self).save(commit=False)
            instance.customer = self.customer
            instance.priority = self.cleaned_data['priority']
            instance.save()
            return instance

    @property
    def address_type(self):
        return self.Meta.model.__name__.lower().rstrip('address')

    @property
    def is_primary(self):
        return app_settings.PRIMARY_ADDRESS == self.address_type


class ShippingAddressForm(AddressForm):
    use_primary_address_label = _('Use billing address for shipping')

    class Meta(AddressForm.Meta):
        model = ShippingAddress


class BillingAddressForm(AddressForm):
    use_primary_address_label = _('Use shipping address for billing')

    class Meta(AddressForm.Meta):
        model = BillingAddress


class PaymentMethodForm(CheckoutFormMixin, forms.Form):
    payment_modifier = forms.ChoiceField(
        label=_('Payment method'),
        widget=forms.RadioSelect,
    )

    def __init__(self, *args, **kwargs):
        super(PaymentMethodForm, self).__init__(*args, **kwargs)
        choices = [x.get_choice() for x in cart_modifiers_pool.get_payment_modifiers() if not x.is_disabled(self.cart)]
        self.fields['payment_modifier'].choices = choices
        if len(choices) == 1:
            self.fields['payment_modifier'].initial = choices[0][0]


class DeliveryMethodForm(CheckoutFormMixin, forms.Form):
    shipping_modifier = forms.ChoiceField(
        label=_('Delivery method'),
        widget=forms.RadioSelect,
    )

    def __init__(self, *args, **kwargs):
        super(DeliveryMethodForm, self).__init__(*args, **kwargs)
        choices = [x.get_choice() for x in cart_modifiers_pool.get_shipping_modifiers()
                   if not x.is_disabled(self.cart)]
        self.fields['shipping_modifier'].choices = choices
        if len(choices) == 1:
            self.fields['shipping_modifier'].initial = choices[0][0]


class ExtraAnnotationForm(CheckoutFormMixin, forms.Form):
    annotation = forms.CharField(
        label=_('Extra annotation for this order'),
        required=False,
        widget=forms.Textarea,
    )


class AcceptConditionForm(CheckoutFormMixin, forms.Form):
    accept = forms.BooleanField(
        label=_('Accept'),
        required=True,
        widget=forms.CheckboxInput,
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-04-05 06:49
from __future__ import unicode_literals

from django.db import migrations, models


def forwards(apps, schema_editor):
    CartItem = apps.get_model('shopit', 'CartItem')
    for item in CartItem.objects.all():
        item.product_code = item.extra.get('product_code', '')
        item.save()


def backwards(apps, schema_editor):
    CartItem = apps.get_model('shopit', 'CartItem')
    for item in CartItem.objects.all():
        item.extra['product_code'] = item.product_code
        item.save()


class Migration(migrations.Migration):

    dependencies = [
        ('shopit', '0004_add_description_to_categorization'),
    ]

    operations = [
        migrations.AddField(
            model_name='cartitem',
            name='product_code',
            field=models.CharField(blank=True, help_text='Product code of added item.', max_length=255, null=True, verbose_name='Product code'),
        ),
        migrations.AlterField(
            model_name='billingaddress',
            name='priority',
            field=models.SmallIntegerField(db_index=True, default=0, help_text='Priority for using this address'),
        ),
        migrations.AlterField(
            model_name='shippingaddress',
            name='priority',
            field=models.SmallIntegerField(db_index=True, default=0, help_text='Priority for using this address'),
        ),
        migrations.RunPython(forwards, reverse_code=backwards),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.10 on 2017-01-19 07:44
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shopit', '0003_rebuild_flag_mptt_tree'),
    ]

    operations = [
        migrations.AddField(
            model_name='brandtranslation',
            name='description',
            field=models.TextField(blank=True, help_text="Description of a categorization, usually used as lead text in categorization's detail view.", verbose_name='Description'),
        ),
        migrations.AddField(
            model_name='categorytranslation',
            name='description',
            field=models.TextField(blank=True, help_text="Description of a categorization, usually used as lead text in categorization's detail view.", verbose_name='Description'),
        ),
        migrations.AddField(
            model_name='manufacturertranslation',
            name='description',
            field=models.TextField(blank=True, help_text="Description of a categorization, usually used as lead text in categorization's detail view.", verbose_name='Description'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import itertools
import math

from django import template
from django.db.models import Q
from django.db.models.query import QuerySet
from django.utils import six
from shop.money import Money
from shop.money.money_maker import MoneyMaker

from shopit.forms.shop import CartDiscountCodeForm
from shopit.models import categorization as categorization_models
from shopit.models.cart import Cart
from shopit.models.flag import Flag
from shopit.models.modifier import Modifier
from shopit.models.order import Order
from shopit.models.product import Attribute, Product

register = template.Library()


@register.filter
def moneyformat(value, currency_code=None):
    """
    Format a number into Money type.
    """
    return MoneyMaker(currency_code)(value)


@register.simple_tag(takes_context=True)
def query_transform(context, *args, **kwargs):
    """
    Appends or updates current query string. Can be used as pairs
    passing in every second arg as value so that key can be dynamic.
    It also supports the kwargs format.
    {% query_transform <key> <value> <key> <value> <key>=<value> %}
    """
    get = context['request'].GET.copy()
    if args:
        args_keys = [args[i] for i in range(len(args)) if i % 2 == 0]
        args_vals = [args[i] for i in range(len(args)) if i % 2 != 0]
        for i in range(len(args_vals)):
            k, v = args_keys[i], args_vals[i]
            if v == '' and k in get:
                del get[k]
            elif v != '':
                get[k] = v
    for k, v in kwargs.items():
        if v == '' and k in get:
            del get[k]
        elif v != '':
            get[k] = v
    return get.urlencode()


@register.simple_tag
def get_products(limit=None, flags=None, categories=0, brands=0, manufacturers=0, price_from=None, price_to=None):
    """
    Returns product queryset. Categorization si marked as `0` by default so
    that products with `None` categorizations can be queried. A comma
    separated list of categorization slugs can be passed in.

    {% get_products 3 categories=3 brands='apple' flags='featured,awesome' as featured_products %}
    """
    filters = {}
    products = Product.objects.active()

    if categories != 0:
        if isinstance(categories, six.string_types):
            products = products.filter_categorization(categories=categories.split(','))
        elif isinstance(categories, QuerySet):
            filters['_category_id__in'] = categories.values_list('id', flat=True)
        else:
            filters['_category'] = categories

    if brands != 0:
        if isinstance(brands, six.string_types):
            products = products.filter_categorization(brands=brands.split(','))
        elif isinstance(brands, QuerySet):
            filters['_brand_id__in'] = brands.values_list('id', flat=True)
        else:
            filters['_brand'] = brands

    if manufacturers != 0:
        if isinstance(manufacturers, six.string_types):
            products = products.filter_categorization(manufacturers=manufacturers.split(','))
        elif isinstance(manufacturers, QuerySet):
            filters['_manufacturer_id__in'] = manufacturers.values_list('id', flat=True)
        else:
            filters['_manufacturer'] = manufacturers

    products = products.filter_flags(flags.split(',') if flags else None)
    products = products.filter_price(price_from, price_to)
    return products.top_level().filter(**filters)[:limit]


@register.simple_tag
def get_categorization(categorization, products=None, limit=None, level=None, depth=None, parent=None):
    """
    Returns a categorization list. First argument must be categorization type.
    If `products` is passed in only return categorization appearing in the given list of products.

    {% get_categorization 'brand' products=product_list limit=3 level=1 depth=2 as brands %}
    """
    if categorization not in ['category', 'brand', 'manufacturer']:
        raise template.TemplateSyntaxError(
            "Tag `get_categorization` requires first argument to be either 'category', 'brand' or 'manufacturer'.")

    filters = {}

    if products is not None:
        filters['id__in'] = list(set([getattr(x, '_%s_id' % categorization) for x in products]))

    if level is not None:
        if depth is not None:
            filters['level__gte'] = level
            filters['level__lt'] = level + depth
        else:
            filters['level'] = level

    if parent is not None:
        filters['parent'] = parent

    return getattr(categorization_models, categorization.capitalize()).objects.active().filter(**filters)[:limit]


@register.simple_tag
def get_flags(code=None, products=None, limit=None, level=None, depth=None, parent=None):
    """
    Returns flag with the given code. If code is None returns all, in that
    case products, limit, level, depth & parent can be passed in to filter the results.
    If `products` is passed in only returns flags appearing in the given list of products.

    {% get_flags 'featured' as featured_flag %}
    {% get_flags products=product_list level=1 parent='featured' as featured_flags %}
    """
    if code is not None:
        return Flag.objects.active().filter(code=code).first()

    flags = Flag.objects.active()

    if products is not None:
        flags = flags.filter(Q(product__in=products) | Q(product__group__isnull=False, product__group__in=products))
        flags = flags.distinct()

    filters = {}

    if level is not None:
        if depth is not None:
            filters['level__gte'] = level
            filters['level__lt'] = level + depth
        else:
            filters['level'] = level

    if parent is not None:
        filters['parent'] = parent

    return flags.filter(**filters)[:limit]


@register.simple_tag
def get_modifiers(code=None, products=None, limit=None, filtering=False):
    """
    Return modifier with the given code. If code is None returns all, in that
    case products, limit & filtering can be passed in to filter the results.
    If `products` is passed in only returns modifiers appearing in the given list of products.
    If `filtering` is set to True, only returns modifiers that are eligible for filtering.

    {% get_modifiers 'special-discount' as special_discount_mod %}
    {% get_modifiers products=product_list filtering=True %}
    """
    if code is not None:
        return Modifier.objects.active().filter(code=code).first()

    filters = {}

    if products is not None:
        product_mods = [x.get_modifiers().values_list('code', flat=True) for x in products]
        filters['code__in'] = list(set(itertools.chain.from_iterable(product_mods)))

    modifiers = Modifier.objects.active()

    if filtering:
        modifiers = modifiers.filtering_enabled()

    return modifiers.filter(**filters)[:limit]


@register.simple_tag
def get_attributes(products=None):
    """
    Returns active attributes. If `products` are passed in returns attributes
    only used in the given list of products.

    {% get_attributes products as attributes %}
    """
    attributes = Attribute.objects.active()

    if products is not None:
        product_ids = [x.id for x in products]
        attr_ids = list(set(Product.objects.filter(group_id__in=product_ids).values_list('attributes', flat=True)))
        attributes = attributes.filter(id__in=attr_ids)
    return attributes


@register.simple_tag
def get_price_steps(steps=5, products=None):
    """
    Returns min and max price with the steps in between.

    {% get_price_steps 3 products as price_steps %}
    """
    queryset = Product.objects.active().top_level()

    if products is not None:
        queryset = queryset.filter(id__in=[x.id for x in products])
    if not queryset:
        return []

    queryset = queryset.order_by('_unit_price')
    min_price, max_price = math.floor(queryset.first().unit_price), math.ceil(queryset.last().unit_price)
    if max_price == min_price:
        return [Money(min_price)]

    price_steps = [min_price]
    chunk = int(max_price - min_price) / (steps + 1)
    for i in range(steps):
        price_steps.append(price_steps[-1] + chunk)
    price_steps = sorted(list(set(price_steps)))
    price_steps.append(max_price)
    price_steps = [Money(x) for x in price_steps]
    return price_steps


@register.inclusion_tag('shopit/includes/add_to_cart.html', takes_context=True)
def add_to_cart(context, product=None, watch=False):
    """
    Renders add to cart template. Either the product needs to be available
    within the context, or a product must be passed in. `watch` can be passed
    in to include a watch button as well.

    {% add_to_cart watch=True %}
    """
    product = product or context.get('product', None)
    if not product:
        raise template.TemplateSyntaxError(
            "Tag `add_to_cart` requires to pass in the `product` or for the product to be available in the context.")
    context.update({'product': product, 'watch': watch})
    return context


@register.inclusion_tag('shopit/includes/cart.html', takes_context=True)
def cart(context, editable=True):
    """
    Renders a cart template for the current request. `editable` boolean
    can be passed in to disable cart edits.

    {% cart editable=False %}
    """
    request = context['request']
    cart = context.get('cart') or Cart.objects.get_or_create_from_request(request)
    cart.update(request)

    return {
        'cart': cart,
        'cart_items': cart._cached_cart_items,
        'editable': editable,
        'form': context.get('form', CartDiscountCodeForm(cart=cart)),
    }


@register.inclusion_tag('shopit/includes/order.html', takes_context=True)
def order(context, order=None, number=None):
    """
    Renders order template for the given order or given order number.
    Uses latest order if both `order` and `number` is not passed in.

    {% order number="2018-00001" %}
    """
    if not order and number:
        order_number = Order.resolve_number(number)['number']
        order = Order.objects.filter(number=order_number).first()
    elif not order:
        order = Order.objects.filter_from_request(context['request']).first()

    return {
        'order': order,
        'order_items': order.items.all() if order else [],
    }



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-04-10 09:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shopit', '0011_set_default_price_to_zero'),
    ]

    operations = [
        migrations.AddField(
            model_name='flag',
            name='template',
            field=models.CharField(blank=True, help_text='You can specify a template for rendering this flag or leave it empty for the default look.', max_length=255, null=True, verbose_name='Template'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.contrib import admin
from shop.admin.customer import CustomerAdminBase, CustomerInlineAdminBase, CustomerProxy


class CustomerInlineAdmin(CustomerInlineAdminBase):
    readonly_fields = ['get_number', 'salutation', 'phone_number']
    fieldsets = [(None, {'fields': ['get_number', 'salutation', 'phone_number']})]


@admin.register(CustomerProxy)
class CustomerAdmin(CustomerAdminBase):
    inlines = [CustomerInlineAdmin]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from shopit.models.cart import Cart, CartItem, CartDiscountCode
from shopit.models.customer import Customer
from shopit.models.address import ShippingAddress, BillingAddress
from shopit.models.order import Order, OrderItem
from shopit.models.delivery import Delivery, DeliveryItem
from shopit.models.tax import Tax
from shopit.models.modifier import Modifier, ModifierCondition, DiscountCode
from shopit.models.flag import Flag
from shopit.models.categorization import Category, Brand, Manufacturer
from shopit.models.product import Product, Attribute, AttributeChoice, AttributeValue, Attachment, Relation, Review


__all__ = ['Cart', 'CartItem', 'CartDiscountCode', 'Customer', 'ShippingAddress', 'BillingAddress', 'Order',
           'OrderItem', 'Delivery', 'DeliveryItem', 'Tax', 'Modifier', 'ModifierCondition', 'DiscountCode', 'Flag',
           'Category', 'Brand', 'Manufacturer', 'Product', 'Attribute', 'AttributeChoice', 'AttributeValue',
           'Attachment', 'Relation', 'Review']



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool
from django.utils.translation import ugettext_lazy as _

from shopit.conf import app_settings
from shopit.urls import get_urls


@apphook_pool.register
class ShopitApphook(CMSApp):
    name = _('Shopit')

    def get_urls(self, page=None, language=None, **kwargs):
        if app_settings.SINGLE_APPHOOK:  # pragma: no cover
            return ['shopit.urls']
        return get_urls('shop')


class ShopitAccountApphook(CMSApp):
    name = _('Shopit Account')

    def get_urls(self, page=None, language=None, **kwargs):
        return get_urls('account')


class ShopitCategoriesApphook(CMSApp):
    name = _('Shopit Categories')

    def get_urls(self, page=None, language=None, **kwargs):
        return get_urls('categories')


class ShopitBrandsApphook(CMSApp):
    name = _('Shopit Brands')

    def get_urls(self, page=None, language=None, **kwargs):
        return get_urls('brands')


class ShopitManufacturersApphook(CMSApp):
    name = _('Shopit Manufacturers')

    def get_urls(self, page=None, language=None, **kwargs):
        return get_urls('manufacturers')


class ShopitProductsApphook(CMSApp):
    name = _('Shopit Products')

    def get_urls(self, page=None, language=None, **kwargs):
        return get_urls('products')


if not app_settings.SINGLE_APPHOOK:  # pragma: no cover
    apphook_pool.register(ShopitAccountApphook)
    apphook_pool.register(ShopitCategoriesApphook)
    apphook_pool.register(ShopitBrandsApphook)
    apphook_pool.register(ShopitManufacturersApphook)
    apphook_pool.register(ShopitProductsApphook)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from adminsortable2.admin import SortableAdminMixin, SortableInlineAdminMixin
from django.contrib import admin
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _
from parler.admin import TranslatableAdmin

from shopit.models.modifier import DiscountCode, Modifier, ModifierCondition


class ModifierConditionInline(SortableInlineAdminMixin, admin.TabularInline):
    model = ModifierCondition
    extra = 0


@admin.register(Modifier)
class ModifierAdmin(SortableAdminMixin, TranslatableAdmin):
    list_display = [
        'name', 'code', 'get_value', 'kind', 'get_requires_code', 'get_filtering_enabled', 'active', 'language_column']

    list_filter = ['kind']
    readonly_fields = ['created_at', 'updated_at', 'get_requires_code_field', 'get_filtering_enabled_field']

    fieldsets = [
        (_('Basic info'), {'fields': ['name', 'code']}),
        (_('Status'), {'fields': [('active', 'created_at', 'updated_at')]}),
        (_('Amounts'), {'fields': ['amount', 'percent']}),
        (_('Settings'), {'fields': ['kind', 'get_requires_code_field', 'get_filtering_enabled_field']}),
    ]

    inlines = [ModifierConditionInline]

    class Media:
        css = {'all': ['shopit/css/djangocms-admin-style.css']}

    def get_prepopulated_fields(self, request, obj=None):
        return {'code': ['name']}  # pragma: no cover

    def get_value(self, obj):
        return '%s %%' % obj.percent if obj.percent else str(obj.amount)
    get_value.short_description = _('Value')

    def get_requires_code(self, obj):
        return obj.requires_code
    get_requires_code.boolean = True
    get_requires_code.short_description = _('Codes')

    def get_filtering_enabled(self, obj):
        return obj.is_filtering_enabled
    get_filtering_enabled.boolean = True
    get_filtering_enabled.short_description = _('Filtering')

    def get_requires_code_field(self, obj):  # pragma: no cover
        html = '<img src="/static/admin/img/icon-yes.svg" alt="True">'
        if not obj.requires_code:
            html = '<img src="/static/admin/img/icon-no.svg" alt="False">'
        help_text = _('Displays if code is required for the modifier to be valid. This value depends on '
                      'active codes that are assigned to this modifier.')
        return format_html('%s<p class="help">%s</p>' % (html, help_text))
    get_requires_code_field.allow_tags = True
    get_requires_code_field.short_description = _('Requires code')

    def get_filtering_enabled_field(self, obj):  # pragma: no cover
        if obj.is_filtering_enabled:
            html = '<img src="/static/admin/img/icon-yes.svg" alt="True">'
        else:
            html = '<img src="/static/admin/img/icon-no.svg" alt="False">'
        help_text = _('Displays if modifier can be used as a filter to return products with this modifier selected. '
                      'Filtering is enabled when modifier is not a "Cart modifier", does not require any codes & '
                      'has no conditions to be met.')
        return format_html('%s<p class="help">%s</p>' % (html, help_text))
    get_filtering_enabled_field.allow_tags = True
    get_filtering_enabled_field.short_description = _('Filtering enabled')


@admin.register(DiscountCode)
class DiscountCodeAdmin(SortableAdminMixin, admin.ModelAdmin):
    list_display = ['code', 'modifier', 'num_uses', 'get_is_valid']
    list_filter = ['modifier']
    raw_id_fields = ['customer']
    readonly_fields = ['get_is_valid_field']

    fieldsets = [
        (_('Basic info'), {'fields': ['code', 'modifier']}),
        (_('Status'), {'fields': ['active']}),
        (_('Settings'), {'fields': ['customer', 'max_uses', 'num_uses', ('valid_from', 'valid_until')]}),
        (None, {'fields': ['get_is_valid_field']}),
    ]

    class Media:
        css = {'all': ['shopit/css/djangocms-admin-style.css']}

    def get_is_valid(self, obj):
        return obj.is_valid
    get_is_valid.boolean = True
    get_is_valid.admin_order_field = 'valid_from'
    get_is_valid.short_description = _('Is valid')

    def get_is_valid_field(self, obj):  # pragma: no cover
        if obj.is_valid:
            html = '<img src="/static/admin/img/icon-yes.svg" alt="True">'
        else:
            html = '<img src="/static/admin/img/icon-no.svg" alt="False">'
        help_text = _('Displays if code is valid by checking that: code is active, times used is less than max '
                      'uses, is within the valid time period.')
        return format_html('%s<p class="help">%s</p>' % (html, help_text))
    get_is_valid_field.allow_tags = True
    get_is_valid_field.short_description = _('Is valid')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from cms.models.fields import PlaceholderField
from cms.utils.i18n import get_current_language
from django.core.urlresolvers import NoReverseMatch, reverse
from django.db import models
from django.db.models import Q
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from filer.fields.image import FilerImageField
from mptt.managers import TreeManager
from mptt.models import MPTTModel, TreeForeignKey
from mptt.querysets import TreeQuerySet
from parler.managers import TranslatableManager, TranslatableQuerySet
from parler.models import TranslatableModelMixin, TranslatedFields
from parler.utils.context import switch_language

from shopit.models.flag import Flag
from shopit.models.modifier import Modifier
from shopit.models.tax import Tax


class CategorizationQuerySet(TranslatableQuerySet, TreeQuerySet):
    def active(self):
        return self.filter(Q(active=True) & (Q(parent__isnull=True) | Q(parent__active=True)))


class CategorizationManager(TranslatableManager, TreeManager):
    queryset_class = CategorizationQuerySet

    def get_queryset(self):
        return self.queryset_class(self.model, using=self._db).order_by('tree_id', 'lft')

    def active(self):
        return self.get_queryset().active()


def _categorization_translated_fields():
    return TranslatedFields(
        name=models.CharField(
            _('Name'),
            max_length=128,
        ),
        slug=models.SlugField(
            _('Slug'),
            db_index=True,
        ),
        description=models.TextField(
            _('Description'),
            blank=True,
            help_text=_("Description of a categorization, usually used as lead text in categorization's detail view."),
        ),
        meta={
            'unique_together': [('language_code', 'slug')],
        },
    )


@python_2_unicode_compatible
class CategorizationModel(TranslatableModelMixin, MPTTModel):
    """
    Categorization model, adds translated fields and uses django-mptt.
    Intended for use like this:
        class Category(CategorizationModel):
            translations = _categorization_translated_fields()
    """
    _featured_image = FilerImageField(
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        verbose_name=_('Featured image'),
        help_text=_("If left empty for childs, a parent's featured image will be used."),
    )

    parent = TreeForeignKey(
        'self',
        models.CASCADE,
        blank=True,
        null=True,
        related_name='children',
        verbose_name=_('Parent'),
    )

    modifiers = models.ManyToManyField(
        Modifier,
        blank=True,
        verbose_name=_('Modifiers'),
        limit_choices_to={'kind__in': [Modifier.STANDARD, Modifier.DISCOUNT]},
    )

    flags = models.ManyToManyField(
        Flag,
        blank=True,
        verbose_name=_('Flags'),
        help_text=_('Check flags for products in this categorization.'),
    )

    active = models.BooleanField(
        _('Active'),
        default=True,
        help_text=_('Is this categorization publicly visible.'),
    )

    created_at = models.DateTimeField(
        _('Created at'),
        auto_now_add=True,
    )

    updated_at = models.DateTimeField(
        _('Updated at'),
        auto_now=True,
    )

    objects = CategorizationManager()

    class Meta:
        abstract = True

    def __str__(self):
        return self.safe_translation_getter('name', any_language=True)

    def get_absolute_url(self, language=None):
        if not language:
            language = get_current_language()  # pragma: no cover

        url_name = 'shopit-%s-detail' % self._meta.model.__name__.lower()

        with switch_language(self, language):
            try:
                return reverse(url_name, args=[self.get_path()])
            except NoReverseMatch:  # pragma: no cover
                pass

    def get_path(self):
        """
        Returns ful url path for categorization object.
        """
        path = []
        for obj in self.get_ancestors(include_self=True):
            path.append(obj.safe_translation_getter('slug', ''))
        return '/'.join(path)

    @property
    def featured_image(self):
        if not self._featured_image and self.is_child_node():
            return self.parent.featured_image
        return self._featured_image

    @featured_image.setter
    def featured_image(self, value):
        self._featured_image = value

    def get_products(self):
        """
        Returns active products from this categorization
        """
        products = getattr(self, '_products', None)
        if products is None:
            products = self.product_set.active()
            for child in self.get_descendants():
                products = products | child.product_set.active()
            setattr(self, '_products', products)
        return products

    def get_modifiers(self, distinct=True):
        """
        Returns all modifiers for the current tree.
        """
        mods = getattr(self, '_mods', None)
        if mods is None:
            mods = self.modifiers.active()
            for parent in self.get_ancestors():
                mods = mods | parent.modifiers.active()
            setattr(self, '_mods', mods)
        return mods.distinct() if distinct else mods

    def get_flags(self, distinct=True):
        """
        Returns all flags for the current tree.
        """
        flags = getattr(self, '_flags', None)
        if flags is None:
            flags = self.flags.active()
            for parent in self.get_ancestors():
                flags = flags | parent.flags.active()
            setattr(self, '_flags', flags)
        return flags.distinct() if distinct else flags


class Category(CategorizationModel):
    translations = _categorization_translated_fields()

    content = PlaceholderField('shopit_category_content')

    _tax = models.ForeignKey(
        Tax,
        models.SET_NULL,
        blank=True,
        null=True,
        verbose_name=_('Tax'),
        help_text=_("Tax to be applied to products in this category. If empty, parent's tax will be used."),
    )

    class Meta:
        db_table = 'shopit_categories'
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')
        ordering = ['tree_id', 'lft']

    @property
    def tax(self):
        if not self._tax and self.parent:
            return self.parent.tax
        return self._tax

    @tax.setter
    def tax(self, value):
        self._tax = value


class Brand(CategorizationModel):
    translations = _categorization_translated_fields()

    content = PlaceholderField('shopit_brand_content')

    class Meta:
        db_table = 'shopit_brands'
        verbose_name = _('Brand')
        verbose_name_plural = _('Brands')
        ordering = ['tree_id', 'lft']


class Manufacturer(CategorizationModel):
    translations = _categorization_translated_fields()

    content = PlaceholderField('shopit_manufacturer_content')

    class Meta:
        db_table = 'shopit_manufacturers'
        verbose_name = _('Manufacturer')
        verbose_name_plural = _('Manufacturers')
        ordering = ['tree_id', 'lft']



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from shopit.conf import app_settings


def get_error_message(key, default=""):
    """
    Returns error message from Shopit's `ERROR_MESSAGES` setting.
    """
    messages = getattr(app_settings, 'ERROR_MESSAGES', {})
    return messages.get(key, default)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from shopit.forms.shop import (CartDiscountCodeForm, CustomerForm, GuestForm, ShippingAddressForm, BillingAddressForm,
                               PaymentMethodForm, DeliveryMethodForm, ExtraAnnotationForm, AcceptConditionForm)
from shopit.forms.categorization import CategoryModelForm, BrandModelForm, ManufacturerModelForm
from shopit.forms.flag import FlagModelForm
from shopit.forms.product import (ProductModelForm, AttributeChoiceInlineFormSet, AttributeValueInlineFormSet,
                                  AttributeValueModelForm)
from shopit.forms.account import AccountLoginForm, AccountRegisterForm, AccountDetailsForm, AccountPasswordForm


__all__ = ['CartDiscountCodeForm', 'CustomerForm', 'GuestForm', 'ShippingAddressForm', 'BillingAddressForm',
           'PaymentMethodForm', 'DeliveryMethodForm', 'ExtraAnnotationForm', 'AcceptConditionForm',
           'CategoryModelForm', 'BrandModelForm', 'ManufacturerModelForm', 'FlagModelForm', 'ProductModelForm',
           'AttributeChoiceInlineFormSet', 'AttributeValueInlineFormSet', 'AttributeValueModelForm',
           'AccountLoginForm', 'AccountRegisterForm', 'AccountDetailsForm', 'AccountPasswordForm']



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.cache import never_cache
from parler.views import ViewUrlMixin
from rest_framework import status
from rest_framework.generics import ListCreateAPIView, RetrieveUpdateDestroyAPIView
from rest_framework.response import Response
from rest_framework.settings import api_settings
from shop.views.catalog import AddToCartView as AddToCartViewBase
from shop.views.catalog import ProductListView as BaseProductListView
from shop.views.catalog import ProductRetrieveView

from shopit.conf import app_settings
from shopit.models.cart import Cart, CartItem
from shopit.models.product import Attribute, Product
from shopit.rest.renderers import ModifiedCMSPageRenderer
from shopit.serializers import (AddToCartSerializer, CartItemSerializer, ProductDetailSerializer,
                                ProductSummarySerializer, ReviewSerializer, WatchItemSerializer)

CATEGORIES_VAR = 'c'
BRANDS_VAR = 'b'
MANUFACTURERS_VAR = 'm'
FLAGS_VAR = 'f'
MODIFIERS_VAR = 'd'
PRICE_FROM_VAR = 'pf'
PRICE_TO_VAR = 'pt'
SORT_VAR = 's'


class ProductListView(BaseProductListView):
    serializer_class = ProductSummarySerializer
    renderer_classes = [ModifiedCMSPageRenderer] + api_settings.DEFAULT_RENDERER_CLASSES

    def get(self, request, *args, **kwargs):
        """
        If products are loaded asynchronously, controlled by
        `ASYNC_PRODUCT_LIST` setting, render template without any data.
        """
        if app_settings.ASYNC_PRODUCT_LIST and request.accepted_renderer.format == 'html':
            return Response({})
        return super(ProductListView, self).get(request, *args, **kwargs)

    def list(self, request, *args, **kwargs):
        """
        Return all products count when `count` exists in GET, applicable
        only when format is not html.
        """
        if request.GET.get('get_count', None) and request.accepted_renderer.format != 'html':
            count = self.filter_queryset(self.get_queryset()).count()
            return Response({'count': count})
        return super(ProductListView, self).list(request, *args, **kwargs)

    def get_queryset(self):
        return Product.objects.translated().active().top_level()

    def filter_queryset(self, queryset):
        queryset = super(ProductListView, self).filter_queryset(queryset)

        categories = list(filter(None, self.request.GET.get(CATEGORIES_VAR, '').split(','))) or None
        brands = list(filter(None, self.request.GET.get(BRANDS_VAR, '').split(','))) or None
        manufacturers = list(filter(None, self.request.GET.get(MANUFACTURERS_VAR, '').split(','))) or None
        queryset = queryset.filter_categorization(categories, brands, manufacturers)

        flags = list(filter(None, self.request.GET.get(FLAGS_VAR, '').split(','))) or None
        queryset = queryset.filter_flags(flags)

        modifiers = list(filter(None, self.request.GET.get(MODIFIERS_VAR, '').split(','))) or None
        queryset = queryset.filter_modifiers(modifiers)

        attrs = Attribute.objects.active()
        attr_codes = attrs.values_list('code', flat=True)
        attr_filters = [(x[0], x[1]) for x in self.request.GET.items() if x[0] in attr_codes]

        # Remove null values from attributes that are not nullable.
        for f in [x for x in attr_filters if not x[1]]:
            if not attrs.get(code=f[0]).nullable:
                attr_filters.remove(f)

        queryset = queryset.filter_attributes(attr_filters)

        price_from = self.request.GET.get(PRICE_FROM_VAR, None)
        price_to = self.request.GET.get(PRICE_TO_VAR, None)
        queryset = queryset.filter_price(price_from, price_to)

        sort = self.request.GET.get(SORT_VAR, None)
        if not sort and app_settings.DEFAULT_PRODUCT_ORDER:
            sort = app_settings.DEFAULT_PRODUCT_ORDER
        sort_map = {
            'name': 'translations__name',
            '-name': '-translations__name',
            'price': '_unit_price',
            '-price': '-_unit_price',
        }
        if sort in sort_map:
            queryset = queryset.order_by(sort_map[sort])

        return queryset

    def get_template_names(self):
        return ['shopit/catalog/product_list.html']

    def get_renderer_context(self):
        """
        Add `product_list` renderer context if format is 'html'. Check against
        `ADD_PRODUCT_LIST_TO_CONTEXT` setting if allowed.
        """
        context = super(ProductListView, self).get_renderer_context()
        if app_settings.ADD_PRODUCT_LIST_TO_CONTEXT and context['request'].accepted_renderer.format == 'html':
            queryset = self.filter_queryset(self.get_queryset())
            page = self.paginate_queryset(queryset)
            if page is not None:
                context.update(self.paginator.get_html_context())
            context['product_list'] = page or queryset
        return context


class ProductDetailView(ViewUrlMixin, ProductRetrieveView):
    serializer_class = ProductDetailSerializer
    renderer_classes = [ModifiedCMSPageRenderer] + api_settings.DEFAULT_RENDERER_CLASSES

    def get(self, request, *args, **kwargs):
        response = super(ProductDetailView, self).get(request, *args, **kwargs)
        if request.accepted_renderer.format == 'html':
            product_id = self.get_object().pk
            menu = request.toolbar.get_or_create_menu('shopit-menu', _('Shopit'))
            menu.add_break()
            menu.add_modal_item(_('Edit Product'), url=reverse('admin:shopit_product_change', args=[product_id]))
            menu.add_sideframe_item(_('Delete Product'), url=reverse('admin:shopit_product_delete', args=[product_id]))
        return response

    def get_object(self):
        if not hasattr(self, '_product'):
            self._product = get_object_or_404(Product.objects.translated(slug=self.kwargs['slug']))
        return self._product

    def get_template_names(self):
        return ['shopit/catalog/product_detail.html']

    def get_view_url(self):
        """
        Return object view url. Used in `get_translated_url` templatetag from parler.
        """
        return self.get_object().get_absolute_url()


class ProductReviewMixin(object):
    """
    Mixin used in product reviews.
    """
    def get_queryset(self, include_inactive=False):
        return self.get_product().get_reviews(language=self.request.LANGUAGE_CODE, include_inactive=include_inactive)

    def get_product(self):
        if not hasattr(self, '_product'):
            self._product = get_object_or_404(Product.objects.translated(slug=self.kwargs.get('slug')))
        return self._product


class ProductReviewListView(ProductReviewMixin, ListCreateAPIView):
    """
    View that handles listing and creatign reviews on a product.
    """
    serializer_class = ReviewSerializer
    renderer_classes = api_settings.DEFAULT_RENDERER_CLASSES

    @method_decorator(never_cache)
    def get(self, request, *args, **kwargs):
        return super(ProductReviewListView, self).get(request, *args, **kwargs)

    def create(self, request, *args, **kwargs):
        """
        Check that customer is registered, and that the review is not already
        written for this product by the registered customer.
        """
        if not request.customer.is_authenticated():
            errors = {'not-registered': [_('Please register to submit a review.')]}
            return Response(errors, status=status.HTTP_400_BAD_REQUEST)
        if self.get_queryset(include_inactive=True).filter(customer=request.customer).exists():
            errors = {'exists': [_('Review already written for this Product.')]}
            return Response(errors, status=status.HTTP_400_BAD_REQUEST)
        data = dict(list(request.data.items()), customer=request.customer, language=request.LANGUAGE_CODE, active=app_settings.REVIEW_ACTIVE_DEFAULT)  # noqa
        serializer = self.get_serializer(data=data)
        serializer.is_valid(raise_exception=True)
        serializer.save(product=self.get_product())
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)


class ProductReviewDetailView(ProductReviewMixin, RetrieveUpdateDestroyAPIView):
    """
    View that handles getting, updating and deleting the review.
    """
    serializer_class = ReviewSerializer
    renderer_classes = api_settings.DEFAULT_RENDERER_CLASSES

    @method_decorator(never_cache)
    def get(self, request, *args, **kwargs):
        return super(ProductReviewDetailView, self).get(request, *args, **kwargs)

    def get_object(self):
        if not hasattr(self, '_object'):
            self._object = get_object_or_404(self.get_queryset(include_inactive=True).filter(id=self.kwargs.get('pk')))
        return self._object

    def update(self, request, *args, **kwargs):
        """
        Only allow update for the review owner.
        """
        if self.get_object().customer != self.request.customer:
            errors = {'not-allowed': [_('You can only update your own reviews.')]}
            return Response(errors, status=status.HTTP_403_FORBIDDEN)
        data = dict(list(request.data.items()), customer=request.customer, language=request.LANGUAGE_CODE)
        serializer = self.get_serializer(self.get_object(), data=data, partial=kwargs.pop('partial', False))
        serializer.is_valid(raise_exception=True)
        serializer.save(product=self.get_product())
        return Response(serializer.data)

    def delete(self, request, *args, **kwargs):
        """
        Only allow delete for the review owner.
        """
        if self.get_object().customer != self.request.customer:
            errors = {'not-allowed': [_('You can only delete your own reviews.')]}
            return Response(errors, status=status.HTTP_403_FORBIDDEN)
        return super(ProductReviewDetailView, self).delete(self.request, *args, **kwargs)


class AddToCartView(AddToCartViewBase):
    """
    View that handles adding product to the cart.
    """
    serializer_class = AddToCartSerializer
    renderer_classes = api_settings.DEFAULT_RENDERER_CLASSES

    def get_context(self, request, **kwargs):
        product = get_object_or_404(Product.objects.translated(slug=self.kwargs['slug']))
        return {'product': product, 'request': request}

    def post(self, request, *args, **kwargs):
        """
        Override to add the product to cart.
        """
        errors = {}
        cart = Cart.objects.get_or_create_from_request(request)
        context = self.get_context(request, **kwargs)
        product = context.pop('product')
        quantity = int(request.data.get('quantity', 1))

        if product.is_group:
            errors['variant'] = [_("You can't add a group product to the cart.")]
        else:
            total_quantity = getattr(product.is_in_cart(cart), 'quantity', 0) + quantity
            available, diff = product.is_available(total_quantity)
            if available:
                item, created = CartItem.objects.get_or_create(
                    cart=cart, product=product, quantity=quantity, product_code=product.product_code)
                serializer_class = WatchItemSerializer if total_quantity == 0 else CartItemSerializer
                serializer = serializer_class(item, context=context)
                return Response(serializer.data, status=status.HTTP_202_ACCEPTED)
            errors['quantity'] = [_('Product not available for given quantity, there is %d left.') % (quantity + diff)]

        return Response(errors, status=status.HTTP_400_BAD_REQUEST)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals


class DefaultSettings(object):  # pragma: no cover
    """
    Default settings for Shopit.
    """
    def _setting(self, name, default=None):
        from django.conf import settings
        return getattr(settings, name, default)

    @property
    def SHOPIT_ERROR_MESSAGES(self):
        """
        A dictionary with error messages used in Shopit.
        """
        from django.utils.translation import ugettext_lazy as _

        default = {
            'duplicate_slug': _("This slug is already used. Try another one."),
            'group_has_group': _("Only variant products have a group."),
            'variant_no_group': _("Variants must have a group selected."),
            'variant_has_category': _("Variant products can't specify categorization. It's inherited from their group."),  # noqa
            'varinat_group_variant': _("Can't set group to variant."),
            'not_group_has_variants': _("This product has variants, you need to delete them before changing it's kind."),  # noqa
            'not_group_has_available_attributes': _("Only group products can have Availible attributes specified."),
            'group_no_available_attributes': _("Group product should have Availible attributes specified."),
            'variant_has_tax': _("Variant products can't specify tax, their group tax percentage will be used instead."),  # noqa
            'variant_no_attributes': _("Variant must have their unique set of attributes specified."),
            'variant_already_exists': _("A Variant with this attributes for selected group already exists."),
            'not_variant_has_attributes': _("Only Variant products can have attributes."),
            'attribute_no_choices': _("Choices must be specified."),
            'attribute_duplicate_choices': _("Attribute can't have duplicate choices."),
            'incorrect_attribute_choice': _("Selected choice doesn't match the seleced attribute."),
            'no_attachment_or_url': _("Missing the attachment or url."),
            'wrong_extension': _("File extension not allowed for this attachment kind."),
            'discount_not_negative': _('A discount should be subtracting the price, amount or percent needs to be negative.'),  # noqa
            'variant_has_relations': _('Only Single and Group products can have relations.'),
            'relation_base_is_product': _("You can't set relation to self."),
            'modifier_no_condition_path': _("You have to select a condition."),
            'cart_discount_code_exists': _("Code is already applied to your cart."),
            'cart_discount_code_invalid': _("Code is invalid or expired."),
            'cart_discount_code_wrong_customer': _("Code is invalid or expired."),
        }
        default.update(self._setting('SHOPIT_ERROR_MESSAGES', {}))
        return default

    @property
    def SHOPIT_ADDRESS_COUNTRIES(self):
        """
        Country choices used in checkout address forms. If empty all countries
        are used from ``shop.models.address.ISO_3166_CODES``.
        """
        return self._setting('SHOPIT_ADDRESS_COUNTRIES', ())

    @property
    def SHOPIT_PRIMARY_ADDRESS(self):
        """
        A primary address to be used in a checkout proccess. Can be 'shipping'
        or 'billing'. Depending on wich address is selected, the other one will
        get the option to use the primary one instead of having to fill it out.
        """
        from django.core.exceptions import ImproperlyConfigured

        value = self._setting('SHOPIT_PRIMARY_ADDRESS', 'shipping')
        if value not in ['shipping', 'billing']:
            raise ImproperlyConfigured("Setting `SHOPIT_PRIMARY_ADDRESS` must be either 'shipping' or 'billing'.")
        return value

    @property
    def SHOPIT_PHONE_NUMBER_REQUIRED(self):
        """
        A flag to control if customer's phone number is required.
        """
        return self._setting('SHOPIT_PHONE_NUMBER_REQUIRED', False)

    @property
    def SHOPIT_PRODUCT_SERIALIZER_FIELDS(self):
        """
        A list of base serializer fields for a common product.
        """
        default = ['id', 'name', 'slug', 'caption', 'code', 'kind', 'url', 'add_to_cart_url', 'price', 'is_available']
        return self._setting('SHOPIT_PRODUCT_SERIALIZER_FIELDS', default)

    @property
    def SHOPIT_PRODUCT_DETAIL_SERIALIZER_FIELDS(self):
        """
        A list of serializer fields for a product detail.
        """
        default = list(set(self.PRODUCT_SERIALIZER_FIELDS + ['variants', 'attributes']))
        return self._setting('SHOPIT_PRODUCT_DETAIL_SERIALIZER_FIELDS', default)

    @property
    def SHOPIT_ATTRIBUTE_TEMPLATES(self):
        """
        Template choices used when rendering an attribute.
        """
        return self._setting('SHOPIT_ATTRIBUTE_TEMPLATES', ())

    @property
    def SHOPIT_RELATION_KINDS(self):
        """
        Relation kind choices on a ``ProductRelation`` model.
        """
        from django.utils.translation import ugettext_lazy as _

        default = (
            ('up-sell', _('Up-sell')),
            ('cross-sell', _('Cross-sell')),
        )
        return self._setting('SHOPIT_RELATION_KINDS', default)

    @property
    def SHOPIT_REVIEW_RATINGS(self):
        """
        Rating choices for product reviews.
        """
        return self._setting('SHOPIT_REVIEW_RATINGS', ())

    @property
    def SHOPIT_REVIEW_ACTIVE_DEFAULT(self):
        """
        Is review active by default when created.
        """
        return self._setting('SHOPIT_REVIEW_ACTIVE_DEFAULT', True)

    @property
    def SHOPIT_ASYNC_PRODUCT_LIST(self):
        """
        A boolean that enables you to optimize ``ProductListView`` and
        ``CategoryDetailView`` when products are fetched asynchronously (ajax).
        """
        return self._setting('SHOPIT_ASYNC_PRODUCT_LIST', False)

    @property
    def SHOPIT_ADD_PRODUCT_LIST_TO_CONTEXT(self):
        """
        A boolean to control if ``product_list`` is added to context when
        accessing a ``ProductListView`` or ``CategoryDetailView``.
        """
        return self._setting('SHOPIT_ADD_PRODUCT_LIST_TO_CONTEXT', not self.SHOPIT_ASYNC_PRODUCT_LIST)

    @property
    def SHOPIT_DEFAULT_PRODUCT_ORDER(self):
        """
        A default product list ordering.
        Must be on of 'name|-name|price|-price'.
        """
        from django.core.exceptions import ImproperlyConfigured

        value = self._setting('SHOPIT_DEFAULT_PRODUCT_ORDER', None)
        if value not in [None, 'name', '-name', 'price', '-price']:
            raise ImproperlyConfigured("Setting `SHOPIT_DEFAULT_PRODUCT_ORDER` must be 'name|-name|price|-price'.")
        return value

    @property
    def SHOPIT_FLAG_TEMPLATES(self):
        """
        Template choices used when rendering a Flag.
        """
        return self._setting('SHOPIT_FLAG_TEMPLATES', ())

    @property
    def SHOPIT_MODIFIER_CONDITIONS(self):
        """
        A list of ``ModifierCondition`` classes that will be used as choices
        for conditions in a Modifier.
        """
        default = [
            'shopit.modifier_conditions.PriceGreaterThanCondition',
            'shopit.modifier_conditions.PriceLessThanCondition',
            'shopit.modifier_conditions.QuantityGreaterThanCondition',
            'shopit.modifier_conditions.QuantityLessThanCondition',
        ]
        return self._setting('SHOPIT_MODIFIER_CONDITIONS', default)

    @property
    def SHOPIT_TEXT_EDITOR(self):
        """
        A text editor widget used to render a rich textarea in Shopit.
        """
        return self._setting('SHOPIT_TEXT_EDITOR', 'djangocms_text_ckeditor.widgets.TextEditorWidget')

    @property
    def SHOPIT_SINGLE_APPHOOK(self):
        """
        Load urls under a single ``ShopitApphook``, or leave the ability to
        add apps separately.
        """
        return self._setting('SHOPIT_SINGLE_APPHOOK', False)

    @property
    def SHOPIT_FILTER_ATTRIBUTES_INCLUDES_VARIANTS(self):
        """
        Designates if products of kind ``VARIANT`` should be included in
        attribute filtered results.
        """
        return self._setting('SHOPIT_FILTER_ATTRIBUTES_INCLUDES_VARIANTS', False)

    def __getattr__(self, key):
        if not key.startswith('SHOPIT_'):
            key = 'SHOPIT_{0}'.format(key)
        return getattr(self, key)


app_settings = DefaultSettings()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class ShopitConfig(AppConfig):
    name = 'shopit'
    verbose_name = _('Shopit')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.contrib import messages
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import PasswordResetForm, SetPasswordForm
from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.urlresolvers import reverse
from django.http import JsonResponse
from django.shortcuts import redirect, render
from django.utils.decorators import method_decorator
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_protect
from django.views.generic import FormView
from rest_framework import status
from rest_framework.generics import GenericAPIView, RetrieveAPIView
from rest_framework.renderers import TemplateHTMLRenderer
from rest_framework.response import Response
from rest_framework.settings import api_settings
from shop.views.auth import LoginView, LogoutView, PasswordResetConfirm, PasswordResetView
from shop.views.order import OrderView

from shopit.forms import account as account_forms
from shopit.models.customer import Customer
from shopit.models.order import Order
from shopit.rest.renderers import ModifiedCMSPageRenderer
from shopit.serializers import (AccountResetConfirmSerializer, AccountResetSerializer, AccountSerializer,
                                OrderListSerializer)


class LoginRegisterMixin(object):
    """
    Redirects already logged in customers to their account.
    """
    def dispatch(self, request, *args, **kwargs):
        if self.request.user.is_authenticated():
            return redirect('shopit-account-detail')
        return super(LoginRegisterMixin, self).dispatch(request, *args, **kwargs)


class AccountLoginView(LoginRegisterMixin, LoginView):
    form_class = account_forms.AccountLoginForm
    template_name = 'shopit/account/account_login.html'

    @method_decorator(never_cache)
    def get(self, request, *args, **kwargs):
        return render(request, self.template_name, {'form': self.form_class()})

    def login(self):
        super(AccountLoginView, self).login()
        messages.success(self.request._request, _('You have been successfully logged in.'))

    @method_decorator(csrf_protect)
    @method_decorator(never_cache)
    def post(self, request, *args, **kwargs):
        # Call super on `LoginView` instead on `AccountLoginView` to skip
        # LoginView's post method and call the original one.
        return super(LoginView, self).post(request, *args, **kwargs)


class AccountLogoutView(LogoutView):
    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        self.post(request, *args, **kwargs)
        messages.success(request, _('You have been successfully logged out.'))
        return redirect('shopit-account-login')

    def post(self, request, *args, **kwargs):
        response = super(AccountLogoutView, self).post(request, *args, **kwargs)
        # Extract message from logout form.
        response.data['success'] = response.data['logout_form']['success_message']
        return response


class AccountResetView(LoginRegisterMixin, PasswordResetView):
    serializer_class = AccountResetSerializer
    form_class = PasswordResetForm
    template_name = 'shopit/account/account_reset.html'

    @method_decorator(never_cache)
    def get(self, request, *args, **kwargs):
        return render(request, self.template_name, {'form': self.form_class()})

    @method_decorator(csrf_protect)
    @method_decorator(never_cache)
    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        if not serializer.is_valid():
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        serializer.save()
        msg = _("Instructions on how to reset the password have been sent to '{email}'.").format(**serializer.data)
        messages.success(request._request, msg)
        return Response({'success': msg}, status=status.HTTP_200_OK)


class AccountResetConfirmView(LoginRegisterMixin, PasswordResetConfirm):
    serializer_class = AccountResetConfirmSerializer
    renderer_classes = [TemplateHTMLRenderer] + api_settings.DEFAULT_RENDERER_CLASSES
    form_class = SetPasswordForm
    template_name = 'shopit/account/account_reset_confirm.html'

    @method_decorator(never_cache)
    def get(self, request, uidb64=None, token=None):
        data = {'uid': uidb64, 'token': token}
        serializer_class = self.get_serializer_class()
        password = get_user_model().objects.make_random_password()
        data.update(new_password1=password, new_password2=password)
        serializer = serializer_class(data=data, context=self.get_serializer_context())
        if not serializer.is_valid():
            return Response({'validlink': False})
        return Response({
            'validlink': True,
            'user_name': force_text(serializer.user),
            'form': self.form_class(serializer.user),
        })

    @method_decorator(csrf_protect)
    @method_decorator(never_cache)
    def post(self, request, uidb64=None, token=None):
        data = dict(list(request.data.items()), uid=uidb64, token=token)
        serializer = self.get_serializer(data=data)
        if not serializer.is_valid():
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        serializer.save()
        msg = _('Account has been reset with the new password.')
        messages.success(request._request, msg)
        return Response({'success': msg})


class AccountRegisterView(LoginRegisterMixin, GenericAPIView):
    form_class = account_forms.AccountRegisterForm
    template_name = 'shopit/account/account_register.html'

    @method_decorator(never_cache)
    def get(self, request, *args, **kwargs):
        return render(request, self.template_name, {'form': self.form_class()})

    @method_decorator(csrf_protect)
    @method_decorator(never_cache)
    def post(self, request, *args, **kwargs):
        if request.customer.is_visitor():
            customer = Customer.objects.get_or_create_from_request(request)
        else:
            customer = request.customer
        form = self.form_class(data=request.data, instance=customer)
        if not form.is_valid():
            return Response(dict(form.errors), status=status.HTTP_400_BAD_REQUEST)
        form.save(request=request)
        msg = _('You have been successfully registered.')
        messages.success(request._request, msg)
        return Response({'success': msg})


class AccountDetailView(LoginRequiredMixin, RetrieveAPIView):
    serializer_class = AccountSerializer
    renderer_classes = [ModifiedCMSPageRenderer] + api_settings.DEFAULT_RENDERER_CLASSES
    template_name = 'shopit/account/account_detail.html'

    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        """
        Add exception to login required mixin when ajax GET request of a non
        visiting customer is in question. This allows to get the Anonymous
        user as JSON.
        """
        if not request.customer.is_visitor() and request.is_ajax() and request.method == 'GET':
            return super(LoginRequiredMixin, self).dispatch(request, *args, **kwargs)
        return super(AccountDetailView, self).dispatch(request, *args, **kwargs)

    def get_object(self):
        return Customer.objects.get_from_request(self.request)


class AccountOrderView(LoginRequiredMixin, OrderView):
    latest = False
    renderer_classes = [ModifiedCMSPageRenderer] + api_settings.DEFAULT_RENDERER_CLASSES
    list_serializer_class = OrderListSerializer
    lookup_field = lookup_url_kwarg = 'pk'

    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        """
        Redirect to latest order if needed.
        """
        if self.latest:
            latest = Order.objects.filter_from_request(request).first()
            return redirect(latest) if latest else redirect('shopit-account-order-list')
        return super(AccountOrderView, self).dispatch(request, *args, **kwargs)

    def get_renderer_context(self):
        context = super(AccountOrderView, self).get_renderer_context()
        if self.request.accepted_renderer.format == 'html':
            if self.many:
                queryset = self.filter_queryset(self.get_queryset())
                page = self.paginate_queryset(queryset)
                if page is not None:
                    context.update(self.paginator.get_html_context())
                context['order_list'] = page or queryset
            else:
                context['order'] = self.get_object()
        return context

    def get_template_names(self):
        if self.many:
            return ['shopit/account/account_order_list.html']
        return ['shopit/account/account_order_detail.html']


class AccountSettingsView(LoginRequiredMixin, FormView):
    """
    Settings view that uses different forms based on `action` property.
    Set 'action = details|password' to access different sections.
    """
    action = None

    @method_decorator(csrf_protect)
    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        return super(AccountSettingsView, self).dispatch(request, *args, **kwargs)

    def form_invalid(self, form):
        if self.request.is_ajax():
            return JsonResponse(dict(form.errors), status=400)
        return super(AccountSettingsView, self).form_invalid(form)

    def form_valid(self, form):
        form.save()
        messages.success(self.request, form.get_success_message())
        if self.request.is_ajax():
            return JsonResponse(form.data)
        return super(AccountSettingsView, self).form_valid(form)

    def get_context_data(self, **kwargs):
        context = {'view': self, 'action': self.action}
        if self.action:
            context['form'] = self.get_form()
        context.update(kwargs)
        return context

    def get_form_class(self):
        return getattr(account_forms, 'Account%sForm' % self.action.capitalize())

    def get_form_kwargs(self):
        kwargs = super(AccountSettingsView, self).get_form_kwargs()
        kwargs['request'] = self.request
        return kwargs

    def get_success_url(self):
        return reverse('shopit-account-settings-%s' % self.action)

    def get_template_names(self):
        names = ['shopit/account/account_settings_%s.html' % self.action] if self.action else []
        names.append('shopit/account/account_settings.html')
        return names

    @property
    def allowed_methods(self):
        allowed = self._allowed_methods()
        if not self.action:
            allowed.remove('POST')
        return allowed



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from shopit.views.shop import CartView, WatchView, CheckoutView, ThanksView
from shopit.views.account import (AccountLoginView, AccountLogoutView, AccountRegisterView, AccountResetView,
                                  AccountResetConfirmView, AccountDetailView, AccountOrderView, AccountSettingsView)
from shopit.views.product import (ProductListView, ProductDetailView, ProductReviewListView, ProductReviewDetailView,
                                  AddToCartView)
from shopit.views.categorization import (CategoryListView, BrandListView, ManufacturerListView,
                                         CategoryDetailView, BrandDetailView, ManufacturerDetailView)


__all__ = ['CartView', 'WatchView', 'CheckoutView', 'ThanksView', 'AccountLoginView', 'AccountLogoutView',
           'AccountRegisterView', 'AccountResetView', 'AccountResetConfirmView', 'AccountDetailView',
           'AccountOrderView', 'AccountSettingsView', 'ProductListView', 'ProductDetailView',
           'ProductReviewListView', 'ProductReviewDetailView', 'AddToCartView',
           'CategoryListView', 'BrandListView', 'ManufacturerListView',
           'CategoryDetailView', 'BrandDetailView', 'ManufacturerDetailView']



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from shop.models.address import ISO_3166_CODES, BaseBillingAddress, BaseShippingAddress


class AddressModelMixin(models.Model):
    name = models.CharField(
        _('Full name'),
        max_length=1024,
    )

    address1 = models.CharField(
        _('Address line 1'),
        max_length=1024,
    )

    address2 = models.CharField(
        _('Address line 2'),
        max_length=1024,
        blank=True,
        null=True,
    )

    zip_code = models.CharField(
        _('ZIP / Postal code'),
        max_length=12,
    )

    city = models.CharField(
        _('City'),
        max_length=1024,
    )

    country = models.CharField(
        _('Country'),
        max_length=3,
        choices=ISO_3166_CODES,
    )

    class Meta:
        abstract = True

    @property
    def label(self):
        fields = [self.name, self.address1, self.address2, self.zip_code, self.city,
                  str(dict(ISO_3166_CODES)[self.country])]
        return ', '.join([x for x in fields if x])


@python_2_unicode_compatible
class ShippingAddress(BaseShippingAddress, AddressModelMixin):
    class Meta:
        db_table = 'shopit_shipping_addresses'
        verbose_name = _('Shipping address')
        verbose_name_plural = _('Shipping addresses')

    def __str__(self):
        return self.label


@python_2_unicode_compatible
class BillingAddress(BaseBillingAddress, AddressModelMixin):
    class Meta:
        db_table = 'shopit_billing_addresses'
        verbose_name = _('Billing address')
        verbose_name_plural = _('Billing addresses')

    def __str__(self):
        return self.label



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import Count, F, Q
from django.utils import timezone
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from parler.managers import TranslatableQuerySet
from parler.models import TranslatableModel, TranslatedFields
from shop.money.fields import MoneyField

from shopit.models.cart import CartDiscountCode
from shopit.models.customer import Customer
from shopit.modifier_conditions import modifier_conditions_pool
from shopit.utils import get_error_message as em


class ModifierQuerySet(TranslatableQuerySet):
    def active(self):
        return self.filter(active=True)

    def filtering_enabled(self):
        """
        Returns queryset with modifiers that are allowed to be used
        when filtering a list of products. This includes modifiers that don't
        have any conditions or discount codes that are required. Also skips
        the 'cart' modifiers.
        """
        return self.filter(kind__in=[Modifier.STANDARD, Modifier.DISCOUNT]).\
            prefetch_related('discount_codes').prefetch_related('conditions').\
            annotate(num_discount_codes=Count('discount_codes'), num_conditions=Count('conditions')).\
            filter(num_discount_codes=0, num_conditions=0)


@python_2_unicode_compatible
class Modifier(TranslatableModel):
    STANDARD = 'standard'
    DISCOUNT = 'discount'
    CART = 'cart'

    KINDS = (
        (STANDARD, _('Standard')),
        (DISCOUNT, _('Discount')),
        (CART, _('Cart')),
    )

    translations = TranslatedFields(
        name=models.CharField(
            _('Name'),
            max_length=128,
        ),
    )

    code = models.SlugField(
        _('Code'),
        unique=True,
        help_text=_('Unique identifier for this modifier.'),
    )

    amount = MoneyField(
        _('Amount'),
        default=0,
        help_text=('Amount that should be added. Can be negative.'),
    )

    percent = models.DecimalField(
        _('Percent'),
        blank=True,
        null=True,
        max_digits=4,
        decimal_places=2,
        help_text=_('Percent that should be added, overrides the amount. Can be negative.'),
    )

    kind = models.CharField(
        _('Kind'),
        max_length=16,
        choices=KINDS,
        default=STANDARD,
        help_text=_(
            'Standard affects the product regardles, Discount checks for a "Discountable" flag on a product and '
            'should be negative, Cart will affect an entire cart.'
        ),
    )

    active = models.BooleanField(
        _('Active'),
        default=True,
        help_text=_('Is this modifier publicly visible.'),
    )

    created_at = models.DateTimeField(
        _('Created at'),
        auto_now_add=True,
    )

    updated_at = models.DateTimeField(
        _('Updated at'),
        auto_now=True,
    )

    order = models.PositiveIntegerField(
        _('Sort'),
        default=0,
    )

    objects = ModifierQuerySet.as_manager()

    class Meta:
        db_table = 'shopit_modifiers'
        verbose_name = _('Modifier')
        verbose_name_plural = _('Modifiers')
        ordering = ['order']

    def __str__(self):
        return self.label

    def save(self, *args, **kwargs):
        self.clean()
        super(Modifier, self).save(*args, **kwargs)

    @property
    def label(self):
        return self.safe_translation_getter('name', any_language=True)

    @property
    def requires_code(self):
        return self.discount_codes.active().exists()

    @property
    def is_filtering_enabled(self):
        return Modifier.objects.filtering_enabled().active().filter(id=self.id).exists()

    def get_conditions(self):
        if not hasattr(self, '_conditions'):
            setattr(self, '_conditions', list(self.conditions.all()))
        return getattr(self, '_conditions')

    def get_discount_codes(self, include_added=False):
        key = '_discount_codes_added' if include_added else '_discount_codes'
        if not hasattr(self, key):
            setattr(self, key, list(self.discount_codes.valid(include_added=include_added)))
        return getattr(self, key)

    def get_added_amount(self, price, quantity=1):
        return self.percent * price / 100 if self.percent else self.amount * quantity

    def can_be_applied(self, request, cart_item=None, cart=None):
        """
        Returns if a modifier can be applied to the given cart or cart item.
        Either `cart_item` or `cart` must be passed in.
        """
        if cart_item is None and cart is None:
            return False

        if cart_item and not self.is_eligible_product(cart_item.product):
            return False

        for condition in self.get_conditions():
            if not condition.is_met(request, cart_item, cart):
                return False

        if self.requires_code and not self.is_code_applied(cart_item.cart_id if cart_item else cart.id):
            return False

        return self.active  # Should never happen to be False up to this point, but just in case.

    def is_eligible_product(self, product):
        """
        Returns if modifier can be applied to the given product.
        """
        if self.kind == self.DISCOUNT:
            return product.discountable
        return self.kind == self.STANDARD

    def is_code_applied(self, cart_id):
        """
        Make sure that at least one code is applied to the given cart.
        """
        cart_codes = CartDiscountCode.objects.filter(cart_id=cart_id).values_list('code', flat=True)
        for code in self.get_discount_codes(include_added=True):
            if code.code in cart_codes:
                return True
        return False

    def clean(self):
        if self.kind == self.DISCOUNT:
            if self.percent and self.percent >= 0 or not self.percent and self.amount >= 0:
                raise ValidationError(em('discount_not_negative'))

    @classmethod
    def get_cart_modifiers(cls):
        return cls.objects.filter(kind=cls.CART)


@python_2_unicode_compatible
class ModifierCondition(models.Model):
    """
    Inline model for Modifier that adds conditions that must be met for
    Modifier to be valid.
    """
    CONDITIONS = modifier_conditions_pool.get_condition_choices()

    modifier = models.ForeignKey(
        Modifier,
        models.CASCADE,
        related_name='conditions',
        verbose_name=_('Modifier'),
    )

    path = models.CharField(
        _('Condition'),
        max_length=255,
        blank=True,
        choices=CONDITIONS,
    )

    value = models.DecimalField(
        _('Value'),
        blank=True,
        null=True,
        max_digits=10,
        decimal_places=2,
    )

    order = models.PositiveIntegerField(
        _('Sort'),
        default=0,
    )

    class Meta:
        db_table = 'shopit_modifier_conditions'
        verbose_name = _('Condition')
        verbose_name_plural = _('Conditions')
        ordering = ['order']

    def __str__(self):
        name = dict(self.CONDITIONS).get(self.path)
        name = '%s %s' % (name, self.value or '')
        return name.rstrip()

    def save(self, *args, **kwargs):
        self.clean()
        super(ModifierCondition, self).save(*args, **kwargs)

    def is_met(self, request, cart_item=None, cart=None):
        if self.condition and cart_item:
            return self.condition.cart_item_condition(request, cart_item, self.value)
        if self.condition and cart:
            return self.condition.cart_condition(request, cart, self.value)
        return True

    def clean(self):
        if not self.path:
            raise ValidationError(em('modifier_no_condition_path'))

    @cached_property
    def condition(self):
        return modifier_conditions_pool.get_condition(self.path)


class DiscountCodeQuerySet(models.QuerySet):
    def active(self):
        return self.filter(active=True)

    def valid(self, include_added=False):
        """
        `include_added` decides if one extra should be added to `max_uses` for
        validating codes already added to cart.
        """
        now = timezone.now()
        qs = self.active().filter(Q(valid_from__lte=now) & (Q(valid_until__isnull=True) | Q(valid_until__gt=now)))
        if include_added:
            return qs.filter(Q(max_uses__isnull=True) | Q(num_uses__lte=F('max_uses')))
        return qs.filter(Q(max_uses__isnull=True) | Q(num_uses__lt=F('max_uses')))


@python_2_unicode_compatible
class DiscountCode(models.Model):
    """
    Discount code model when added to the modifier, it requires it to also
    be added to the cart.
    """
    modifier = models.ForeignKey(
        Modifier,
        models.CASCADE,
        related_name='discount_codes',
        verbose_name=_('Modifier'),
        help_text=_('Modifier that this discount code applies to.'),
    )

    code = models.CharField(
        _('Code'),
        max_length=30,
        unique=True,
        help_text=_('Code that must be entered for the modifier to activate.'),
    )

    customer = models.ForeignKey(
        Customer,
        models.CASCADE,
        blank=True,
        null=True,
        related_name='discount_codes',
        verbose_name=_('Customer'),
        help_text=_('Limit code so that it can be used only by a specific customer.'),
    )

    max_uses = models.PositiveIntegerField(
        _('Max uses'),
        blank=True,
        null=True,
        help_text=_('Number of times this code can be used, leave empty for unlimited usage.'),
    )

    num_uses = models.PositiveIntegerField(
        _('Num uses'),
        default=0,
        help_text=_('Number of times this code has been already used.'),
    )

    active = models.BooleanField(
        _('Active'),
        default=True,
        help_text=_('Is this discount code active.'),
    )

    valid_from = models.DateTimeField(
        _('Valid from'),
        default=timezone.now,
    )

    valid_until = models.DateTimeField(
        _('Valid until'),
        blank=True,
        null=True,
    )

    order = models.PositiveIntegerField(
        _('Sort'),
        default=0,
    )

    objects = DiscountCodeQuerySet.as_manager()

    class Meta:
        db_table = 'shopit_discount_codes'
        verbose_name = _('Discount code')
        verbose_name_plural = _('Discount codes')
        ordering = ['order']

    def __str__(self):
        return self.code

    @property
    def is_valid(self):
        now = timezone.now()
        if not self.active:
            return False
        if self.max_uses is not None and self.num_uses >= self.max_uses:
            return False
        if self.valid_until:
            return self.valid_from <= now and self.valid_until > now
        return self.valid_from <= now

    def use(self, times=1):
        """
        Should be called when code is used to update `num_uses` field.
        """
        self.num_uses = self.num_uses + times
        self.save(update_fields=['num_uses'])



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.contrib import messages
from django.http import HttpResponseBadRequest, JsonResponse
from django.shortcuts import redirect
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_protect
from django.views.generic import FormView, TemplateView
from shop.modifiers.pool import cart_modifiers_pool

from shopit.forms import shop as shop_forms
from shopit.models.cart import Cart, CartItem
from shopit.models.modifier import DiscountCode
from shopit.models.order import Order


class CartObjectMixin(object):
    """
    Cart object mixin adds cart, cart items and watch items to the context.
    Resets the values from `extra` dict that are populated by the checkout.
    """
    def dispatch(self, request, *args, **kwargs):
        self.cart = Cart.objects.get_or_create_from_request(request)
        for key in ['payment_modifier', 'shipping_modifier', 'annotation']:
            self.cart.extra.pop(key, None)
        self.update_cart()
        self.cart.save()
        return super(CartObjectMixin, self).dispatch(request, *args, **kwargs)

    def update_cart(self):
        self.cart._dirty = True
        self.cart._cached_cart_items = None
        self.cart.update(self.request)

    def get_cart_data(self):
        return {
            'cart': self.cart,
            'cart_items': self.cart._cached_cart_items,
            'watch_items': CartItem.objects.filter_watch_items(self.cart, self.request),
        }

    def get_context_data(self, **kwargs):
        context = super(CartObjectMixin, self).get_context_data(**kwargs)
        context.update(self.get_cart_data())
        return context


class CartView(CartObjectMixin, FormView):
    """
    Cart view displays the cart and handles updating item's quantity,
    deleting the cart and adding modifier codes to the cart.
    """
    empty = False
    form_class = shop_forms.CartDiscountCodeForm
    template_name = 'shopit/shop/cart.html'

    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        if self.empty:
            cart = Cart.objects.get_from_request(request)
            for item in cart.items.all():
                item.delete()
            codes = []
            for code in cart.get_discount_codes():
                codes.append(code.code)
                code.delete()
            for dc in DiscountCode.objects.filter(code__in=codes):
                dc.use(-1)
        return super(CartView, self).dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        code = form.cleaned_data.get('code', None)
        if code and self.request.POST.get('validate', None):
            msg = _('Discount code is valid.')
        else:
            if code:
                form.save()
                msg = _('Discount code has been applied successfully.')
            else:
                msg = _('Cart has been updated successfully.')
            messages.success(self.request, msg)
        if self.request.is_ajax():
            return JsonResponse({'success': msg})
        return redirect('shopit-cart')

    def form_invalid(self, form):
        if self.request.is_ajax():
            return JsonResponse(dict(form.errors), status=400)
        return super(CartView, self).form_invalid(form)

    def get_form_kwargs(self):
        kwargs = super(CartView, self).get_form_kwargs()
        kwargs['cart'] = self.cart
        return kwargs

    @method_decorator(csrf_protect)
    def post(self, request, *args, **kwargs):
        for item, quantity in [x for x in request.POST.items() if x[0].startswith('quantity') and x[1]]:
            item, quantity = int(item.split('-').pop()), int(quantity)
            item = CartItem.objects.get(pk=item)
            if quantity > 0:
                available, diff = item.product.is_available(quantity)
                if available:
                    item.quantity = quantity
                else:
                    item.quantity = quantity + diff
                item.save()
            else:
                item.delete()
        self.update_cart()
        return super(CartView, self).post(request, *args, **kwargs)


class WatchView(CartObjectMixin, TemplateView):
    template_name = 'shopit/shop/watch.html'

    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        return super(WatchView, self).dispatch(request, *args, **kwargs)


class CheckoutView(CartObjectMixin, FormView):
    """
    Checkout view that handles customer selection forms and redirects to
    purchase url of selected payment provider.
    """
    template_name = 'shopit/shop/checkout.html'

    def forms_valid(self, **forms):
        """
        All the forms are valid, make the purchase happen. Return's JSON.
        """
        forms['customer_form'].save()
        self.cart.shipping_address = forms['shipping_form'].save()
        self.cart.billing_address = forms['billing_form'].save()
        self.cart.extra.update(forms['payment_form'].cleaned_data)
        self.cart.extra.update(forms['delivery_form'].cleaned_data)
        self.cart.extra.update(forms['extra_form'].cleaned_data)
        self.update_cart()
        self.cart.save()
        for modifier in cart_modifiers_pool.get_payment_modifiers():
            if modifier.is_active(self.cart):
                payment_provider = getattr(modifier, 'payment_provider', None)
                if payment_provider:
                    expression = payment_provider.get_payment_request(self.cart, self.request)
                    return JsonResponse({'expression': expression})
        return HttpResponseBadRequest()

    def forms_invalid(self, **forms):
        self.cart.save()
        errors = dict([('%s-%s' % (x.prefix, y[0]), y[1]) for x in forms.values() for y in x.errors.items()])
        return JsonResponse(errors, status=400)

    @method_decorator(never_cache)
    def get(self, request, *args, **kwargs):
        response = super(CheckoutView, self).get(request, *args, **kwargs)
        return response if not self.cart.is_empty else redirect('shopit-cart')

    def get_context_data(self, **kwargs):
        context = {'view': self}
        context.update(self.get_cart_data())
        context.update(self.get_forms())
        context.update(kwargs)
        return context

    def get_forms(self, **kwargs):
        kwargs['request'] = self.request
        kwargs['cart'] = self.cart
        forms = {
            'shipping_form': shop_forms.ShippingAddressForm(prefix='shipping', **kwargs),
            'billing_form': shop_forms.BillingAddressForm(prefix='billing', **kwargs),
            'payment_form': shop_forms.PaymentMethodForm(prefix='payment', **kwargs),
            'delivery_form': shop_forms.DeliveryMethodForm(prefix='delivery', **kwargs),
            'extra_form': shop_forms.ExtraAnnotationForm(prefix='extra', **kwargs),
            'accept_form': shop_forms.AcceptConditionForm(prefix='accept', **kwargs),
        }
        if self.request.customer.is_registered():
            forms['customer_form'] = shop_forms.CustomerForm(prefix='customer', **kwargs)
        else:
            forms['customer_form'] = shop_forms.GuestForm(prefix='guest', **kwargs)
        return forms

    @method_decorator(csrf_protect)
    @method_decorator(never_cache)
    def post(self, request, *args, **kwargs):
        """
        Ment to be accessed via ajax, since the `get_payment_request` method
        from the payment provider returns a javascript expression that needs
        to be evaluated within javascript.
        """
        if not request.is_ajax():
            return HttpResponseBadRequest()

        forms = self.get_forms(data=request.POST)
        if all([x.is_valid() for x in forms.values()]):
            return self.forms_valid(**forms)
        return self.forms_invalid(**forms)


class ThanksView(TemplateView):
    """
    A generic thank you view, adds last updated order to the context,
    redirects to cart if no order.
    """
    template_name = 'shopit/shop/thanks.html'

    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        self.order = Order.objects.filter_from_request(request).first()
        if not self.order:
            return redirect('shopit-cart')
        return super(ThanksView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(ThanksView, self).get_context_data(**kwargs)
        context['order'] = self.order
        return context



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import sys
import shopit

from setuptools import find_packages, setup


CLASSIFIERS = [
    'Environment :: Web Environment',
    'Framework :: Django :: 1.11',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
]

setup(
    author='Dino Perovic',
    author_email='dino.perovic@gmail.com',
    name='django-shopit',
    version=shopit.__version__,
    description='Fully featured shop application built on djangoSHOP framework.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/dinoperovic/django-shopit',
    license='BSD License',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    packages=find_packages(exclude=['tests', 'docs']),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'django-shop==0.12.3',
        'django-cms==3.5.3',
        'django-parler>=1.6.5',
        'django-mptt>=0.8.6',
        'django-admin-sortable2>=0.6.4',
        'django-measurement>=2.4.0',
        'djangocms-cascade==0.17.10',
        'djangorestframework==3.8.2',
    ],
    setup_requires=['pytest-runner'] if {'pytest', 'test', 'ptr'}.intersection(sys.argv) else [],
    tests_require=['pytest-django'],
)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-17 10:07
from __future__ import unicode_literals

import cms.models.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '__first__'),
        ('shopit', '0009_add_max_and_num_uses_to_discount_code'),
    ]

    operations = [
        migrations.AddField(
            model_name='brand',
            name='content',
            field=cms.models.fields.PlaceholderField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, slotname='shopit_brand_content', to='cms.Placeholder'),
        ),
        migrations.AddField(
            model_name='category',
            name='content',
            field=cms.models.fields.PlaceholderField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, slotname='shopit_category_content', to='cms.Placeholder'),
        ),
        migrations.AddField(
            model_name='manufacturer',
            name='content',
            field=cms.models.fields.PlaceholderField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, slotname='shopit_manufacturer_content', to='cms.Placeholder'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.contrib import admin
from shop.admin.delivery import DeliveryOrderAdminMixin
from shop.admin.order import OrderAdmin as OrderAdminBase
from shop.admin.order import PrintOrderAdminMixin

from shopit.models.order import Order


@admin.register(Order)
class OrderAdmin(PrintOrderAdminMixin, DeliveryOrderAdminMixin, OrderAdminBase):
    class Media:
        css = {'all': ['shopit/css/djangocms-admin-style.css']}



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _
from django_fsm import transition


class ShippingWorkflowMixin(object):
    """
    A simple workflow mixin that adds a shipping state transtion to the order.
    """
    TRANSITION_TARGETS = {
        'ship_goods': _('Ship goods'),
    }

    @transition(field='status', source=['payment_confirmed'], target='ship_goods',
                custom=dict(admin=True, button_name=_('Ship the goods')))
    def ship_goods(self, by=None):
        pass



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-12-19 13:33
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('shopit', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='flag',
            options={'ordering': ['tree_id', 'lft'], 'verbose_name': 'Flag', 'verbose_name_plural': 'Flags'},
        ),
        migrations.RemoveField(
            model_name='flag',
            name='order',
        ),
        migrations.AddField(
            model_name='flag',
            name='level',
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='flag',
            name='lft',
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='flag',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='shopit.Flag', verbose_name='Parent'),
        ),
        migrations.AddField(
            model_name='flag',
            name='rght',
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='flag',
            name='tree_id',
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.core.urlresolvers import reverse
from shop.payment.defaults import ForwardFundPayment as ForwardFundPaymentBase


class ForwardFundPayment(ForwardFundPaymentBase):
    """
    Modified ForwardFundPayment to use regular javascript to redirect,
    instead of angularJS.
    """
    def get_payment_request(self, cart, request):
        super(ForwardFundPayment, self).get_payment_request(cart, request)
        return 'window.location.href="{}";'.format(reverse('shopit-thanks'))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.utils.module_loading import import_string
from mptt.forms import MPTTAdminForm
from parler.forms import TranslatableModelForm

from shopit.conf import app_settings
from shopit.models.categorization import Brand, Category, Manufacturer

try:
    TextEditor = import_string(app_settings.TEXT_EDITOR)
except ImportError:  # pragma: no cover
    from django.forms.widgets import Textarea as TextEditor


class CategoryModelForm(MPTTAdminForm, TranslatableModelForm):
    class Meta:
        model = Category
        exclude = []
        widgets = {'description': TextEditor()}


class BrandModelForm(MPTTAdminForm, TranslatableModelForm):
    class Meta:
        model = Brand
        exclude = []
        widgets = {'description': TextEditor()}


class ManufacturerModelForm(MPTTAdminForm, TranslatableModelForm):
    class Meta:
        model = Manufacturer
        exclude = []
        widgets = {'description': TextEditor()}



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from shop.models.delivery import BaseDelivery, BaseDeliveryItem


@python_2_unicode_compatible
class Delivery(BaseDelivery):
    class Meta:
        db_table = 'shopit_deliveries'
        verbose_name = _('Delivery')
        verbose_name_plural = _('Deliveries')

    def __str__(self):
        return str(self.pk)


@python_2_unicode_compatible
class DeliveryItem(BaseDeliveryItem):
    quantity = models.IntegerField(
        _('Delivered quantity'),
        default=0,
    )

    class Meta:
        db_table = 'shopit_delivery_items'
        verbose_name = _('Delivery item')
        verbose_name_plural = _('Delivery items')

    def __str__(self):
        return self.item.product_name



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from shopit.admin.customer import CustomerAdmin
from shopit.admin.order import OrderAdmin
from shopit.admin.tax import TaxAdmin
from shopit.admin.modifier import ModifierAdmin, DiscountCodeAdmin
from shopit.admin.flag import FlagAdmin
from shopit.admin.categorization import CategoryAdmin, BrandAdmin, ManufacturerAdmin
from shopit.admin.product import AttributeAdmin, ProductAdmin


__all__ = ['CustomerAdmin', 'OrderAdmin', 'TaxAdmin', 'ModifierAdmin', 'DiscountCodeAdmin', 'FlagAdmin',
           'CategoryAdmin', 'BrandAdmin', 'ManufacturerAdmin', 'AttributeAdmin', 'ProductAdmin']



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.core.validators import MinValueValidator
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from shop.models.cart import BaseCart, BaseCartItem

from shopit.models.address import BillingAddress, ShippingAddress


@python_2_unicode_compatible
class Cart(BaseCart):
    shipping_address = models.ForeignKey(
        ShippingAddress,
        null=True,
        default=None,
        related_name='+',
    )

    billing_address = models.ForeignKey(
        BillingAddress,
        null=True,
        default=None,
        related_name='+',
    )

    class Meta:
        db_table = 'shopit_carts'
        verbose_name = _('Shopping cart')
        verbose_name_plural = _('Shopping carts')

    def __str__(self):
        return str(self.pk)

    def get_discount_codes(self):
        return self.discount_codes.all()

    def delete_discount_codes(self):
        self.get_discount_codes().delete()


@python_2_unicode_compatible
class CartItem(BaseCartItem):
    quantity = models.IntegerField(validators=[MinValueValidator(0)])

    class Meta:
        db_table = 'shopit_cart_items'
        verbose_name = _('Cart item')
        verbose_name_plural = _('Cart items')

    def __str__(self):
        return self.product.product_name


@python_2_unicode_compatible
class CartDiscountCode(models.Model):
    """
    Discount codes that are added to the cart.
    """
    cart = models.ForeignKey(
        Cart,
        models.CASCADE,
        related_name='discount_codes',
        editable=False,
    )

    code = models.CharField(
        _('Code'),
        max_length=30,
    )

    class Meta:
        db_table = 'shopit_cart_discount_codes'
        verbose_name = _('Cart discount code')
        verbose_name_plural = _('Cart discount codes')

    def __str__(self):
        return self.code



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-03 09:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shopit', '0008_add_phone_number_to_customer'),
    ]

    operations = [
        migrations.AddField(
            model_name='discountcode',
            name='max_uses',
            field=models.PositiveIntegerField(blank=True, help_text='Number of times this code can be used, leave empty for unlimited usage.', null=True, verbose_name='Max uses'),
        ),
        migrations.AddField(
            model_name='discountcode',
            name='num_uses',
            field=models.PositiveIntegerField(default=0, help_text='Number of times this code has been already used.', verbose_name='Num uses'),
        ),
        migrations.AlterField(
            model_name='discountcode',
            name='active',
            field=models.BooleanField(default=True, help_text='Is this discount code active.', verbose_name='Active'),
        ),
    ]
